
# Copyright (c) 2013 Huawei Technologies Co., Ltd.
# Copyright (c) 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Unit Tests for Huawei T and Dorado volume drivers.
"""

import mox
import os
import shutil
import socket
import tempfile
import time

from xml.dom.minidom import Document
from xml.etree import ElementTree as ET

from cinder import context
from cinder import exception
from cinder import test
from cinder import utils
from cinder.volume import configuration as conf
from cinder.volume.drivers.huawei import huawei_utils
from cinder.volume.drivers.huawei import HuaweiVolumeDriver
from cinder.volume.drivers.huawei import ssh_common
from cinder.volume import volume_types


LUN_INFO = {'ID': None,
            'Name': None,
            'Size': None,
            'LUN WWN': None,
            'Status': None,
            'Visible Capacity': None,
            'Disk Pool ID': None,
            'Cache Prefetch Strategy': None,
            'Lun Type': None,
            'Consumed Capacity': None,
            'Pool ID': None,
            'SnapShot ID': None,
            'LunCopy ID': None,
            'Owner Controller': None,
            'Worker Controller': None,
            'RAID Group ID': None}

CLONED_LUN_INFO = {'ID': None,
                   'Name': None,
                   'Size': None,
                   'LUN WWN': None,
                   'Status': None,
                   'Visible Capacity': None,
                   'Disk Pool ID': None,
                   'Cache Prefetch Strategy': None,
                   'Lun Type': None,
                   'Consumed Capacity': None,
                   'Pool ID': None,
                   'SnapShot ID': None,
                   'LunCopy ID': None,
                   'Owner Controller': None,
                   'Worker Controller': None,
                   'RAID Group ID': None}

SNAPSHOT_INFO = {'Source LUN ID': None,
                 'Source LUN Name': None,
                 'ID': None,
                 'Name': None,
                 'Type': 'Public',
                 'Status': None}

MAP_INFO = {'Host Group ID': None,
            'Host Group Name': None,
            'Host ID': None,
            'Host Name': None,
            'Os Type': None,
            'INI Port ID': None,
            'INI Port Name': None,
            'INI Port Info': None,
            'INI Port WWN': None,
            'INI Port Type': None,
            'Link Status': None,
            'LUN WWN': None,
            'DEV LUN ID': None,
            'Host LUN ID': None,
            'CHAP status': False}

HOST_PORT_INFO = {'ID': None,
                  'Name': None,
                  'Info': None,
                  'WWN': None,
                  'Type': None}

LUNCOPY_INFO = {'Name': None,
                'ID': None,
                'Type': None,
                'State': None,
                'Status': None}

LUNCOPY_SETTING = {'ID': '1',
                   'Type': 'FULL',
                   'State': 'Created',
                   'Status': 'Normal'}

POOL_SETTING = {'ID': '2',
                'Level': 'RAID6',
                'Status': 'Normal',
                'Free Capacity': '10240',
                'Disk List': '0,1;0,2;0,3;0,4;0,5;0,6',
                'Name': 'RAID_001',
                'Type': 'Thick'}

INITIATOR_SETTING = {'TargetIQN': 'iqn.2006-08.com.huawei:oceanspace:2103037:',
                     'TargetIQN-form': 'iqn.2006-08.com.huawei:oceanspace:'
                     '2103037::1020001:192.168.100.2',
                     'Initiator Name': 'iqn.1993-08.debian:01:ec2bff7ac3a3',
                     'Initiator TargetIP': '192.168.100.2',
                     'WWN': ['2011666666666565']}

FAKE_VOLUME = {'name': 'Volume-lele34fe-223f-dd33-4423-asdfghjklqwe',
               'id': 'lele34fe-223f-dd33-4423-asdfghjklqwe',
               'size': '2',
               'provider_auth': None,
               'volume_type_id': None,
               'provider_location': None}

FAKE_CLONED_VOLUME = {'name': 'Volume-jeje34fe-223f-dd33-4423-asdfghjklqwg',
                      'id': 'jeje34fe-223f-dd33-4423-asdfghjklqwg',
                      'size': '3',
                      'provider_auth': None,
                      'volume_type_id': None,
                      'provider_location': None}

FAKE_SNAPSHOT = {'name': 'keke34fe-223f-dd33-4423-asdfghjklqwf',
                 'id': '223f-dd33-4423-asdfghjklqwf',
                 'volume_name': 'Volume-lele34fe-223f-dd33-4423-asdfghjklqwe',
                 'provider_location': None}

FAKE_CONNECTOR = {'initiator': 'iqn.1993-08.debian:01:ec2bff7ac3a3',
                  'wwpns': ['1000000164s45126'],
                  'wwnns': ['2000666666666565'],
                  'host': 'fakehost',
                  'ip': '10.10.0.1'}

RESPOOL_A_SIM = {'Size': '10240', 'Valid Size': '5120'}
RESPOOL_B_SIM = {'Size': '10240', 'Valid Size': '10240'}
VOLUME_SNAP_ID = {'vol': '0', 'vol_copy': '1', 'snap': '2'}

cmd_error_list = []  # CLI cmds in this list will run failed
Curr_test = ['']  # show current testing driver


class FakeChannel():
    def __init__(self):
        if Curr_test[0] == 'T':
            self.simu = HuaweiTCLIResSimulator()
        elif Curr_test[0] == 'Dorado5100':
            self.simu = HuaweiDorado5100CLIResSimulator()
        else:
            self.simu = HuaweiDorado2100G2CLIResSimulator()

    def resize_pty(self, width=80, height=24):
        pass

    def settimeout(self, time):
        pass

    def send(self, s):
        self.command = s

    def recv(self, nbytes):
        command = self.command.split()
        cmd = command[0]
        params = command[1:]
        if cmd in cmd_error_list:
            reset_error_flg(cmd)
            out = self.command[:-1] + 'ERROR' + '\nadmin:/>'
            return out.replace('\n', '\r\n')
        func_name = 'cli_' + cmd
        cli_func = getattr(self.simu, func_name)
        out = cli_func(params)
        out = self.command[:-1] + out + '\nadmin:/>'
        return out.replace('\n', '\r\n')

    def close(self):
        pass


class FakeSSHClient():
    def invoke_shell(self):
        return FakeChannel()

    def get_transport(self):

        class transport():
            def __init__(self):
                self.sock = sock()

        class sock():
            def settimeout(self, time):
                pass

        return transport()

    def close(self):
        pass


class FakeSSHPool():
    def __init__(self, ip, port, conn_timeout, login, password=None,
                 *args, **kwargs):
        self.ip = ip
        self.port = port
        self.login = login
        self.password = password

    def create(self):
        return FakeSSHClient()

    def get(self):
        return FakeSSHClient()

    def put(self, ssh):
        pass

    def remove(self, ssh):
        pass


def Fake_sleep(time):
    pass


def Fake_change_file_mode(obj, filepath):
    pass


def create_fake_conf_file(filename):
    doc = Document()

    config = doc.createElement('config')
    doc.appendChild(config)

    storage = doc.createElement('Storage')
    config.appendChild(storage)
    product = doc.createElement('Product')
    product_text = doc.createTextNode('T')
    product.appendChild(product_text)
    storage.appendChild(product)
    config.appendChild(storage)
    protocol = doc.createElement('Protocol')
    protocol_text = doc.createTextNode('iSCSI')
    protocol.appendChild(protocol_text)
    storage.appendChild(protocol)
    controllerip0 = doc.createElement('ControllerIP0')
    controllerip0_text = doc.createTextNode('10.10.10.1')
    controllerip0.appendChild(controllerip0_text)
    storage.appendChild(controllerip0)
    controllerip1 = doc.createElement('ControllerIP1')
    controllerip1_text = doc.createTextNode('10.10.10.2')
    controllerip1.appendChild(controllerip1_text)
    storage.appendChild(controllerip1)
    username = doc.createElement('UserName')
    username_text = doc.createTextNode('admin')
    username.appendChild(username_text)
    storage.appendChild(username)
    userpassword = doc.createElement('UserPassword')
    userpassword_text = doc.createTextNode('123456')
    userpassword.appendChild(userpassword_text)
    storage.appendChild(userpassword)

    lun = doc.createElement('LUN')
    config.appendChild(lun)
    storagepool = doc.createElement('StoragePool')
    storagepool.setAttribute('Name', 'RAID_001')
    lun.appendChild(storagepool)
    luntype = doc.createElement('LUNType')
    luntype_text = doc.createTextNode('Thick')
    luntype.appendChild(luntype_text)
    lun.appendChild(luntype)

    iscsi = doc.createElement('iSCSI')
    config.appendChild(iscsi)
    defaulttargetip = doc.createElement('DefaultTargetIP')
    defaulttargetip_text = doc.createTextNode('192.168.100.1')
    defaulttargetip.appendChild(defaulttargetip_text)
    iscsi.appendChild(defaulttargetip)
    initiator = doc.createElement('Initiator')
    initiator.setAttribute('Name', 'iqn.1993-08.debian:01:ec2bff7ac3a3')
    initiator.setAttribute('TargetIP', '192.168.100.2')
    iscsi.appendChild(initiator)

    os_type = doc.createElement('Host')
    os_type.setAttribute('OSType', 'Linux')
    os_type.setAttribute('HostIP', '10.10.0.1')
    config.appendChild(os_type)

    tmp_file = open(filename, 'w')
    tmp_file.write(doc.toprettyxml(indent=''))
    tmp_file.close()


def modify_conf(conf, item, val, attrib=None):
    tree = ET.parse(conf)
    root = tree.getroot()
    conf_item = root.find('%s' % item)
    if not attrib:
        conf_item.text = '%s' % val
    else:
        conf_item.attrib['%s' % attrib] = '%s' % val
    tree.write(conf, 'UTF-8')


def set_error_flg(cmd):
    cmd_error_list.append(cmd)


def reset_error_flg(cmd):
    cmd_error_list.remove(cmd)


class HuaweiTCLIResSimulator():
    def _paras_name(self, params):
        index = params.index('-n')
        return params[index + 1]

    def cli_showsys(self, params):
        pass

    def cli_createlun(self, params):
        lun_type = ('THIN' if '-pool' in params else 'THICK')
        if LUN_INFO['ID'] is None:
            LUN_INFO['Name'] = self._paras_name(params)
            LUN_INFO['ID'] = VOLUME_SNAP_ID['vol']
            LUN_INFO['Size'] = FAKE_VOLUME['size']
            LUN_INFO['Lun Type'] = lun_type
            LUN_INFO['Owner Controller'] = 'A'
            LUN_INFO['Worker Controller'] = 'A'
            LUN_INFO['RAID Group ID'] = POOL_SETTING['ID']
            FAKE_VOLUME['provider_location'] = LUN_INFO['ID']
        else:
            CLONED_LUN_INFO['Name'] = self._paras_name(params)
            CLONED_LUN_INFO['ID'] = VOLUME_SNAP_ID['vol_copy']
            CLONED_LUN_INFO['Size'] = FAKE_CLONED_VOLUME['size']
            CLONED_LUN_INFO['Lun Type'] = lun_type
            CLONED_LUN_INFO['Owner Controller'] = 'A'
            CLONED_LUN_INFO['Worker Controller'] = 'A'
            CLONED_LUN_INFO['RAID Group ID'] = POOL_SETTING['ID']
            FAKE_CLONED_VOLUME['provider_location'] = CLONED_LUN_INFO['ID']
        out = 'command operates successfully'
        return out

    def cli_showlun(self, params):
        if '-lun' not in params:
            if LUN_INFO['ID'] is None:
                out = 'command operates successfully, but no information.'
            elif CLONED_LUN_INFO['ID'] is None:
                out = """/>showlun
===========================================================================
                           LUN Information
---------------------------------------------------------------------------
  ID  RAID Group ID  Disk Pool ID  Status  Controller  Visible Capacity(MB) \
  LUN Name   Stripe Unit Size(KB)   Lun Type
---------------------------------------------------------------------------
  %s    %s    --    Normal    %s    %s    %s    64    THICK
===========================================================================
""" % (LUN_INFO['ID'], LUN_INFO['RAID Group ID'], LUN_INFO['Owner Controller'],
       str(int(LUN_INFO['Size']) * 1024), LUN_INFO['Name'])
            else:
                out = """/>showlun
============================================================================
                               LUN Information
----------------------------------------------------------------------------
  ID  RAID Group ID  Disk Pool ID  Status  Controller  Visible Capacity(MB)\
  LUN Name   Stripe Unit Size(KB)   Lun Type
----------------------------------------------------------------------------
  %s    %s    --    Normal    %s    %s    %s    64    THICK
  %s    %s    --    Normal    %s    %s    %s    64    THICK
============================================================================
""" % (LUN_INFO['ID'], LUN_INFO['RAID Group ID'], LUN_INFO['Owner Controller'],
       str(int(LUN_INFO['Size']) * 1024), LUN_INFO['Name'],
       CLONED_LUN_INFO['ID'], CLONED_LUN_INFO['RAID Group ID'],
       CLONED_LUN_INFO['Owner Controller'],
       str(int(CLONED_LUN_INFO['Size']) * 1024),
       CLONED_LUN_INFO['Name'])

        elif params[params.index('-lun') + 1] in VOLUME_SNAP_ID.values():
            out = """/>showlun
================================================
                 LUN Information
------------------------------------------------
  ID                     |  %s
  Name                   |  %s
  LUN WWN                |  --
  Visible Capacity       |  %s
  RAID GROUP ID          |  %s
  Owning Controller      |  %s
  Workong Controller     |  %s
  Lun Type               |  %s
  SnapShot ID            |  %s
  LunCopy ID             |  %s
================================================
""" % ((LUN_INFO['ID'], LUN_INFO['Name'], LUN_INFO['Visible Capacity'],
        LUN_INFO['RAID Group ID'], LUN_INFO['Owner Controller'],
        LUN_INFO['Worker Controller'], LUN_INFO['Lun Type'],
        LUN_INFO['SnapShot ID'], LUN_INFO['LunCopy ID'])
       if params[params.index('-lun') + 1] == VOLUME_SNAP_ID['vol'] else
       (CLONED_LUN_INFO['ID'], CLONED_LUN_INFO['Name'],
        CLONED_LUN_INFO['Visible Capacity'], CLONED_LUN_INFO['RAID Group ID'],
        CLONED_LUN_INFO['Owner Controller'],
        CLONED_LUN_INFO['Worker Controller'],
        CLONED_LUN_INFO['Lun Type'], CLONED_LUN_INFO['SnapShot ID'],
        CLONED_LUN_INFO['LunCopy ID']))
        else:
            out = 'ERROR: The object does not exist.'
        return out

    def cli_dellun(self, params):
        if params[params.index('-lun') + 1] == VOLUME_SNAP_ID['vol']:
            LUN_INFO['Name'] = None
            LUN_INFO['ID'] = None
            LUN_INFO['Size'] = None
            LUN_INFO['Lun Type'] = None
            LUN_INFO['LUN WWN'] = None
            LUN_INFO['Owner Controller'] = None
            LUN_INFO['Worker Controller'] = None
            LUN_INFO['RAID Group ID'] = None
            FAKE_VOLUME['provider_location'] = None
        else:
            CLONED_LUN_INFO['Name'] = None
            CLONED_LUN_INFO['ID'] = None
            CLONED_LUN_INFO['Size'] = None
            CLONED_LUN_INFO['Lun Type'] = None
            CLONED_LUN_INFO['LUN WWN'] = None
            CLONED_LUN_INFO['Owner Controller'] = None
            CLONED_LUN_INFO['Worker Controller'] = None
            CLONED_LUN_INFO['RAID Group ID'] = None
            CLONED_LUN_INFO['provider_location'] = None
            FAKE_CLONED_VOLUME['provider_location'] = None
        out = 'command operates successfully'
        return out

    def cli_showrg(self, params):
        out = """/>showrg
=====================================================================
                      RAID Group Information
---------------------------------------------------------------------
  ID    Level    Status    Free Capacity(MB)    Disk List    Name
---------------------------------------------------------------------
  0     RAID6    Normal    1024                 0,0;0,2;     RAID003
  %s    %s       %s        %s                   %s           %s
=====================================================================
-""" % (POOL_SETTING['ID'], POOL_SETTING['Level'],
        POOL_SETTING['Status'], POOL_SETTING['Free Capacity'],
        POOL_SETTING['Disk List'], POOL_SETTING['Name'])
        return out

    def cli_showpool(self, params):
        out = """/>showpool
=====================================================================
                      Pool Information
---------------------------------------------------------------------
  Level    Status    Available Capacity(MB)    Disk List
---------------------------------------------------------------------
  RAID6    Normal    %s                        0,0;0,2;0,4;0,5;
=====================================================================
-""" % POOL_SETTING['Free Capacity']
        return out

    def cli_createluncopy(self, params):
        src_id = params[params.index('-slun') + 1]
        tgt_id = params[params.index('-tlun') + 1]
        LUNCOPY_INFO['Name'] = 'OpenStack_%s_%s' % (src_id, tgt_id)
        LUNCOPY_INFO['ID'] = LUNCOPY_SETTING['ID']
        LUNCOPY_INFO['Type'] = LUNCOPY_SETTING['Type']
        LUNCOPY_INFO['State'] = LUNCOPY_SETTING['State']
        LUNCOPY_INFO['Status'] = LUNCOPY_SETTING['Status']
        out = 'command operates successfully'
        return out

    def cli_chgluncopystatus(self, params):
        LUNCOPY_INFO['State'] = 'Start'
        out = 'command operates successfully'
        return out

    def cli_showluncopy(self, params):
        if LUNCOPY_INFO['State'] == 'Start':
            LUNCOPY_INFO['State'] = 'Copying'
        elif LUNCOPY_INFO['State'] == 'Copying':
            LUNCOPY_INFO['State'] = 'Complete'
        out = """/>showluncopy
============================================================================
                            LUN Copy Information
----------------------------------------------------------------------------
  LUN Copy Name    LUN Copy ID    Type    LUN Copy State    LUN Copy Status
----------------------------------------------------------------------------
  %s               %s             %s      %s                %s
============================================================================
""" % (LUNCOPY_INFO['Name'], LUNCOPY_INFO['ID'], LUNCOPY_INFO['Type'],
       LUNCOPY_INFO['State'], LUNCOPY_INFO['Status'])
        return out

    def cli_delluncopy(self, params):
        LUNCOPY_INFO['Name'] = None
        LUNCOPY_INFO['ID'] = None
        LUNCOPY_INFO['Type'] = None
        LUNCOPY_INFO['State'] = None
        LUNCOPY_INFO['Status'] = None
        out = 'command operates successfully'
        return out

    def cli_createsnapshot(self, params):
        SNAPSHOT_INFO['Source LUN ID'] = LUN_INFO['ID']
        SNAPSHOT_INFO['Source LUN Name'] = LUN_INFO['Name']
        SNAPSHOT_INFO['ID'] = VOLUME_SNAP_ID['snap']
        SNAPSHOT_INFO['Name'] = self._paras_name(params)
        SNAPSHOT_INFO['Status'] = 'Disable'
        out = 'command operates successfully'
        return out

    def cli_showsnapshot(self, params):
        if SNAPSHOT_INFO['ID'] is None:
            out = 'command operates successfully, but no information.'
        else:
            out = """/>showsnapshot
==========================================================================
                             Snapshot Information
--------------------------------------------------------------------------
  Name         ID         Type         Status        Time Stamp
--------------------------------------------------------------------------
  %s           %s         Public       %s            2013-01-15 14:21:13
==========================================================================
""" % (SNAPSHOT_INFO['Name'], SNAPSHOT_INFO['ID'], SNAPSHOT_INFO['Status'])
        return out

    def cli_actvsnapshot(self, params):
        SNAPSHOT_INFO['Status'] = 'Active'
        FAKE_SNAPSHOT['provider_location'] = SNAPSHOT_INFO['ID']
        out = 'command operates successfully'
        return out

    def cli_disablesnapshot(self, params):
        SNAPSHOT_INFO['Status'] = 'Disable'
        out = 'command operates successfully'
        return out

    def cli_delsnapshot(self, params):
        SNAPSHOT_INFO['Source LUN ID'] = None
        SNAPSHOT_INFO['Source LUN Name'] = None
        SNAPSHOT_INFO['ID'] = None
        SNAPSHOT_INFO['Name'] = None
        SNAPSHOT_INFO['Status'] = None
        FAKE_SNAPSHOT['provider_location'] = None
        out = 'command operates successfully'
        return out

    def cli_showrespool(self, params):
        out = """/>showrespool
===========================================================================
                         Resource Pool Information
---------------------------------------------------------------------------
  Pool ID    Size(MB)    Usage(MB)    Valid Size(MB)    Alarm Threshold
---------------------------------------------------------------------------
  A          %s          0.0          %s                80
  B          %s          0.0         %s                80
===========================================================================
-""" % (RESPOOL_A_SIM['Size'], RESPOOL_A_SIM['Valid Size'],
        RESPOOL_B_SIM['Size'], RESPOOL_B_SIM['Valid Size'])
        return out

    def cli_showiscsitgtname(self, params):
        iqn = INITIATOR_SETTING['TargetIQN']
        out = """/>showiscsitgtname
===================================================================
                    ISCSI Name
-------------------------------------------------------------------
  Iscsi Name     | %s
===================================================================
""" % iqn
        return out

    def cli_showiscsiip(self, params):
        out = """/>showiscsiip
============================================================================
                          iSCSI IP Information
----------------------------------------------------------------------------
  Controller ID   Interface Module ID   Port ID   IP Address   Mask
----------------------------------------------------------------------------
  B               0                     P1        %s           255.255.255.0
============================================================================
-""" % INITIATOR_SETTING['Initiator TargetIP']
        return out

    def cli_showhostgroup(self, params):
        if MAP_INFO['Host Group ID'] is None:
            out = """/>showhostgroup
============================================================
                   Host Group Information
------------------------------------------------------------
  Host Group ID    Name                File Engine Cluster
------------------------------------------------------------
  0                Default Group       NO
============================================================
"""
        else:
            out = """/>showhostgroup
============================================================
                   Host Group Information
------------------------------------------------------------
  Host Group ID    Name                File Engine Cluster
------------------------------------------------------------
  0                Default Group       NO
  %s               %s                  NO
============================================================
""" % (MAP_INFO['Host Group ID'], MAP_INFO['Host Group Name'])
        return out

    def cli_createhostgroup(self, params):
        MAP_INFO['Host Group ID'] = '1'
        MAP_INFO['Host Group Name'] = 'HostGroup_OpenStack'
        out = 'command operates successfully'
        return out

    def cli_showhost(self, params):
        if MAP_INFO['Host ID'] is None:
            out = 'command operates successfully, but no information.'
        else:
            out = """/>showhost
=======================================================
                   Host Information
-------------------------------------------------------
  Host ID    Host Name      Host Group ID    Os Type
-------------------------------------------------------
  %s         %s             %s               Linux
=======================================================
""" % (MAP_INFO['Host ID'], MAP_INFO['Host Name'], MAP_INFO['Host Group ID'])
        return out

    def cli_addhost(self, params):
        MAP_INFO['Host ID'] = '1'
        MAP_INFO['Host Name'] = 'Host_' + FAKE_CONNECTOR['host']
        MAP_INFO['Os Type'] = 'Linux'
        out = 'command operates successfully'
        return out

    def cli_delhost(self, params):
        MAP_INFO['Host ID'] = None
        MAP_INFO['Host Name'] = None
        MAP_INFO['Os Type'] = None
        out = 'command operates successfully'
        return out

    def cli_showiscsiini(self, params):
        if HOST_PORT_INFO['ID'] is None:
            out = 'Error: The parameter is wrong.'
        else:
            out = """/>showiscsiini
========================================================
                 Initiator Information
--------------------------------------------------------
  Initiator Name                     Chap Status
--------------------------------------------------------
  %s                                 Disable
========================================================
""" % HOST_PORT_INFO['Info']
        return out

    def cli_addiscsiini(self, params):
        HOST_PORT_INFO['ID'] = '1'
        HOST_PORT_INFO['Name'] = 'iSCSIInitiator001'
        HOST_PORT_INFO['Info'] = INITIATOR_SETTING['Initiator Name']
        HOST_PORT_INFO['Type'] = 'ISCSITGT'
        out = 'command operates successfully'
        return out

    def cli_deliscsiini(self, params):
        HOST_PORT_INFO['ID'] = None
        HOST_PORT_INFO['Name'] = None
        HOST_PORT_INFO['Info'] = None
        HOST_PORT_INFO['Type'] = None
        out = 'command operates successfully'
        return out

    def cli_showhostport(self, params):
        if MAP_INFO['INI Port ID'] is None:
            out = 'command operates successfully, but no information.'
        else:
            out = """/>showhostport
============================================================================
                        Host Port Information
----------------------------------------------------------------------------
Port ID  Port Name  Port Information  Port Type  Host ID  Link Status \
Multipath Type
----------------------------------------------------------------------------
 %s      %s       %s       %s       %s       Unconnected       Default
============================================================================
""" % (MAP_INFO['INI Port ID'], MAP_INFO['INI Port Name'],
       MAP_INFO['INI Port Info'], MAP_INFO['INI Port Type'],
       MAP_INFO['Host ID'])
        return out

    def cli_addhostport(self, params):
        MAP_INFO['INI Port ID'] = HOST_PORT_INFO['ID']
        MAP_INFO['INI Port Name'] = HOST_PORT_INFO['Name']
        MAP_INFO['INI Port Info'] = HOST_PORT_INFO['Info']
        MAP_INFO['INI Port Type'] = HOST_PORT_INFO['Type']
        out = 'command operates successfully'
        return out

    def cli_delhostport(self, params):
        MAP_INFO['INI Port ID'] = None
        MAP_INFO['INI Port Name'] = None
        MAP_INFO['INI Port Info'] = None
        MAP_INFO['INI Port Type'] = None
        HOST_PORT_INFO['ID'] = None
        HOST_PORT_INFO['Name'] = None
        HOST_PORT_INFO['Info'] = None
        HOST_PORT_INFO['Type'] = None
        out = 'command operates successfully'
        return out

    def cli_showhostmap(self, params):
        if MAP_INFO['DEV LUN ID'] is None:
            out = 'command operates successfully, but no information.'
        else:
            out = """/>showhostmap
===========================================================================
                           Map Information
---------------------------------------------------------------------------
  Map ID   Working Controller   Dev LUN ID   LUN WWN   Host LUN ID  Mapped to\
  RAID ID   Dev LUN Cap(MB)   Map Type   Whether Command LUN   Pool ID
----------------------------------------------------------------------------
  2147483649   %s   %s   %s   %s   Host: %s   %s   %s   HOST   No   --
============================================================================
""" % (LUN_INFO['Worker Controller'], LUN_INFO['ID'], LUN_INFO['LUN WWN'],
       MAP_INFO['Host LUN ID'], MAP_INFO['Host ID'], LUN_INFO['RAID Group ID'],
       str(int(LUN_INFO['Size']) * 1024))
        return out

    def cli_addhostmap(self, params):
        MAP_INFO['DEV LUN ID'] = LUN_INFO['ID']
        MAP_INFO['LUN WWN'] = LUN_INFO['LUN WWN']
        MAP_INFO['Host LUN ID'] = '2'
        MAP_INFO['Link Status'] = 'Linked'
        out = 'command operates successfully'
        return out

    def cli_delhostmap(self, params):
        if MAP_INFO['Link Status'] == 'Linked':
            MAP_INFO['Link Status'] = 'Deleting'
            out = 'there are IOs accessing the system, please try later'
        else:
            MAP_INFO['Link Status'] = None
            MAP_INFO['DEV LUN ID'] = None
            MAP_INFO['LUN WWN'] = None
            MAP_INFO['Host LUN ID'] = None
            out = 'command operates successfully'
        return out

    def cli_showfreeport(self, params):
        out = """/>showfreeport
=======================================================================
                      Host Free Port Information
-----------------------------------------------------------------------
  WWN Or MAC          Type    Location              Connection Status
-----------------------------------------------------------------------
  1000000164s45126    FC      Primary Controller    Connected
=======================================================================
"""
        HOST_PORT_INFO['ID'] = '2'
        HOST_PORT_INFO['Name'] = 'FCInitiator001'
        HOST_PORT_INFO['Info'] = '1000000164s45126'
        HOST_PORT_INFO['Type'] = 'FC'
        return out

    def cli_showhostpath(self, params):
        host = params[params.index('-host') + 1]
        out = """/>showhostpath -host 1
=======================================
        Multi Path Information
---------------------------------------
  Host ID           | %s
  Controller ID     | B
  Port Type         | FC
  Initiator WWN     | 1000000164s45126
  Target WWN        | %s
  Host Port ID      | 0
  Link Status       | Normal
=======================================
""" % (host, INITIATOR_SETTING['WWN'][0])
        return out

    def cli_showfcmode(self, params):
        out = """/>showfcport
=========================================================================
                      FC Port Topology Mode
-------------------------------------------------------------------------
  Controller ID   Interface Module ID   Port ID   WWN    Current Mode
-------------------------------------------------------------------------
  B               1                     P0        %s     --
=========================================================================
-""" % INITIATOR_SETTING['WWN'][0]
        return out

    def cli_chglun(self, params):
        if params[params.index('-lun') + 1] == VOLUME_SNAP_ID['vol']:
            LUN_INFO['Owner Controller'] = 'B'
        else:
            CLONED_LUN_INFO['Owner Controller'] = 'B'
        out = 'command operates successfully'
        return out

    def cli_addluntoextlun(self, params):
        LUN_INFO['Size'] = int(LUN_INFO['Size']) + int(CLONED_LUN_INFO['Size'])
        out = 'command operates successfully'
        return out

    def cli_rmlunfromextlun(self, patams):
        LUN_INFO['Size'] = int(LUN_INFO['Size']) - int(CLONED_LUN_INFO['Size'])
        out = 'command operates successfully'
        return out


class HuaweiDorado5100CLIResSimulator(HuaweiTCLIResSimulator):
    def cli_showsys(self, params):
        out = """/>showsys
=============================================================
                                System Information
-------------------------------------------------------------
  System Name           | SN_Dorado5100
  Device Type           | Oceanstor Dorado5100
  Current System Mode   | Double Controllers Normal
  Mirroring Link Status | Link Up
  Location              |
  Time                  | 2013-01-01 01:01:01
  Product Version       | V100R001C00
=============================================================
"""
        return out

    def cli_showlun(self, params):
        if '-lun' not in params:
            if LUN_INFO['ID'] is None:
                out = 'command operates successfully, but no information.'
            elif CLONED_LUN_INFO['ID'] is None:
                out = """/>showlun
===========================================================================
                           LUN Information
---------------------------------------------------------------------------
  ID   RAIDgroup ID  Status   Controller  Visible Capacity(MB)   LUN Name..\
  Strip Unit Size(KB)   Lun Type
---------------------------------------------------------------------------
  %s      %s       Normal       %s      %s       %s       64       THICK
===========================================================================
""" % (LUN_INFO['ID'], LUN_INFO['RAID Group ID'],
       LUN_INFO['Owner Controller'], str(int(LUN_INFO['Size']) * 1024),
       LUN_INFO['Name'])
            else:
                out = """/>showlun
===========================================================================
                           LUN Information
---------------------------------------------------------------------------
  ID   RAIDgroup ID   Status   Controller   Visible Capacity(MB)   LUN Name \
  Strip Unit Size(KB)   Lun Type
---------------------------------------------------------------------------
  %s      %s       Normal      %s      %s       %s        64       THICK
  %s      %s       Norma       %s      %s       %s        64       THICK
===========================================================================
""" % (LUN_INFO['ID'], LUN_INFO['RAID Group ID'], LUN_INFO['Owner Controller'],
       str(int(LUN_INFO['Size']) * 1024), LUN_INFO['Name'],
       CLONED_LUN_INFO['ID'], CLONED_LUN_INFO['RAID Group ID'],
       CLONED_LUN_INFO['Owner Controller'],
       str(int(CLONED_LUN_INFO['Size']) * 1024),
       CLONED_LUN_INFO['Name'])
        elif params[params.index('-lun') + 1] in VOLUME_SNAP_ID.values():
            out = """/>showlun
================================================
                 LUN Information
------------------------------------------------
  ID                     |  %s
  Name                   |  %s
  LUN WWN                |  --
  Visible Capacity       |  %s
  RAID GROUP ID          |  %s
  Owning Controller      |  %s
  Workong Controller     |  %s
  Lun Type               |  %s
  SnapShot ID            |  %s
  LunCopy ID             |  %s
================================================
""" % ((LUN_INFO['ID'], LUN_INFO['Name'], LUN_INFO['Visible Capacity'],
        LUN_INFO['RAID Group ID'], LUN_INFO['Owner Controller'],
        LUN_INFO['Worker Controller'], LUN_INFO['Lun Type'],
        LUN_INFO['SnapShot ID'], LUN_INFO['LunCopy ID'])
       if params[params.index('-lun') + 1] == VOLUME_SNAP_ID['vol'] else
       (CLONED_LUN_INFO['ID'], CLONED_LUN_INFO['Name'],
        CLONED_LUN_INFO['Visible Capacity'], CLONED_LUN_INFO['RAID Group ID'],
        CLONED_LUN_INFO['Owner Controller'],
        CLONED_LUN_INFO['Worker Controller'],
        CLONED_LUN_INFO['Lun Type'], CLONED_LUN_INFO['SnapShot ID'],
        CLONED_LUN_INFO['LunCopy ID']))
        else:
            out = 'ERROR: The object does not exist.'
        return out


class HuaweiDorado2100G2CLIResSimulator(HuaweiTCLIResSimulator):
    def cli_showsys(self, params):
        out = """/>showsys
==========================================================================
                                System Information
--------------------------------------------------------------------------
  System Name           | SN_Dorado2100_G2
  Device Type           | Oceanstor Dorado2100 G2
  Current System Mode   | Double Controllers Normal
  Mirroring Link Status | Link Up
  Location              |
  Time                  | 2013-01-01 01:01:01
  Product Version       | V100R001C00
===========================================================================
"""
        return out

    def cli_createlun(self, params):
        lun_type = ('THIN' if params[params.index('-type') + 1] == '2' else
                    'THICK')
        if LUN_INFO['ID'] is None:
            LUN_INFO['Name'] = self._paras_name(params)
            LUN_INFO['ID'] = VOLUME_SNAP_ID['vol']
            LUN_INFO['Size'] = FAKE_VOLUME['size']
            LUN_INFO['Lun Type'] = lun_type
            LUN_INFO['Owner Controller'] = 'A'
            LUN_INFO['Worker Controller'] = 'A'
            LUN_INFO['RAID Group ID'] = POOL_SETTING['ID']
            FAKE_VOLUME['provider_location'] = LUN_INFO['ID']
        else:
            CLONED_LUN_INFO['Name'] = self._paras_name(params)
            CLONED_LUN_INFO['ID'] = VOLUME_SNAP_ID['vol_copy']
            CLONED_LUN_INFO['Size'] = FAKE_CLONED_VOLUME['size']
            CLONED_LUN_INFO['Lun Type'] = lun_type
            CLONED_LUN_INFO['Owner Controller'] = 'A'
            CLONED_LUN_INFO['Worker Controller'] = 'A'
            CLONED_LUN_INFO['RAID Group ID'] = POOL_SETTING['ID']
            CLONED_LUN_INFO['provider_location'] = CLONED_LUN_INFO['ID']
            FAKE_CLONED_VOLUME['provider_location'] = CLONED_LUN_INFO['ID']
        out = 'command operates successfully'
        return out

    def cli_showlun(self, params):
        if '-lun' not in params:
            if LUN_INFO['ID'] is None:
                out = 'command operates successfully, but no information.'
            elif CLONED_LUN_INFO['ID'] is None:
                out = """/>showlun
===========================================================================
                           LUN Information
---------------------------------------------------------------------------
  ID   Status   Controller  Visible Capacity(MB)   LUN Name  Lun Type
---------------------------------------------------------------------------
  %s   Normal   %s          %s                     %s        THICK
===========================================================================
""" % (LUN_INFO['ID'], LUN_INFO['Owner Controller'],
       str(int(LUN_INFO['Size']) * 1024), LUN_INFO['Name'])
            else:
                out = """/>showlun
===========================================================================
                           LUN Information
---------------------------------------------------------------------------
  ID   Status   Controller  Visible Capacity(MB)   LUN Name  Lun Type
---------------------------------------------------------------------------
  %s   Normal   %s          %s                     %s         THICK
  %s   Normal   %s          %s                     %s         THICK
===========================================================================
""" % (LUN_INFO['ID'], LUN_INFO['Owner Controller'],
       str(int(LUN_INFO['Size']) * 1024), LUN_INFO['Name'],
       CLONED_LUN_INFO['ID'], CLONED_LUN_INFO['Owner Controller'],
       str(int(CLONED_LUN_INFO['Size']) * 1024), CLONED_LUN_INFO['Name'])

        elif params[params.index('-lun') + 1] in VOLUME_SNAP_ID.values():
            out = """/>showlun
================================================
                 LUN Information
------------------------------------------------
  ID                     |  %s
  Name                   |  %s
  LUN WWN                |  --
  Visible Capacity       |  %s
  RAID GROUP ID          |  %s
  Owning Controller      |  %s
  Workong Controller     |  %s
  Lun Type               |  %s
  SnapShot ID            |  %s
  LunCopy ID             |  %s
================================================
""" % ((LUN_INFO['ID'], LUN_INFO['Name'], LUN_INFO['Visible Capacity'],
        LUN_INFO['RAID Group ID'], LUN_INFO['Owner Controller'],
        LUN_INFO['Worker Controller'], LUN_INFO['Lun Type'],
        LUN_INFO['SnapShot ID'], LUN_INFO['LunCopy ID'])
       if params[params.index('-lun')] == VOLUME_SNAP_ID['vol'] else
       (CLONED_LUN_INFO['ID'], CLONED_LUN_INFO['Name'],
        CLONED_LUN_INFO['Visible Capacity'], CLONED_LUN_INFO['RAID Group ID'],
        CLONED_LUN_INFO['Owner Controller'],
        CLONED_LUN_INFO['Worker Controller'],
        CLONED_LUN_INFO['Lun Type'], CLONED_LUN_INFO['SnapShot ID'],
        CLONED_LUN_INFO['LunCopy ID']))

        else:
            out = 'ERROR: The object does not exist.'

        return out


class HuaweiTISCSIDriverTestCase(test.TestCase):
    def __init__(self, *args, **kwargs):
        super(HuaweiTISCSIDriverTestCase, self).__init__(*args, **kwargs)

    def setUp(self):
        super(HuaweiTISCSIDriverTestCase, self).setUp()

        self.tmp_dir = tempfile.mkdtemp()
        self.fake_conf_file = self.tmp_dir + '/cinder_huawei_conf.xml'
        create_fake_conf_file(self.fake_conf_file)
        self.configuration = mox.MockObject(conf.Configuration)
        self.configuration.cinder_huawei_conf_file = self.fake_conf_file
        self.configuration.append_config_values(mox.IgnoreArg())

        self.stubs.Set(time, 'sleep', Fake_sleep)
        self.stubs.Set(utils, 'SSHPool', FakeSSHPool)
        self.stubs.Set(ssh_common.TseriesCommon, '_change_file_mode',
                       Fake_change_file_mode)
        self._init_driver()

    def _init_driver(self):
        Curr_test[0] = 'T'
        self.driver = HuaweiVolumeDriver(configuration=self.configuration)
        self.driver.do_setup(None)

    def tearDown(self):
        if os.path.exists(self.fake_conf_file):
            os.remove(self.fake_conf_file)
        shutil.rmtree(self.tmp_dir)
        super(HuaweiTISCSIDriverTestCase, self).tearDown()

    def test_conf_invalid(self):
        # Test config file not found
        tmp_fonf_file = '/xxx/cinder_huawei_conf.xml'
        tmp_configuration = mox.MockObject(conf.Configuration)
        tmp_configuration.cinder_huawei_conf_file = tmp_fonf_file
        tmp_configuration.append_config_values(mox.IgnoreArg())
        self.assertRaises(IOError,
                          HuaweiVolumeDriver,
                          configuration=tmp_configuration)
        # Test Product and Protocol invalid
        tmp_dict = {'Storage/Product': 'T', 'Storage/Protocol': 'iSCSI'}
        for k, v in tmp_dict.items():
            modify_conf(self.fake_conf_file, k, 'xx')
            self.assertRaises(exception.InvalidInput,
                              HuaweiVolumeDriver,
                              configuration=self.configuration)
            modify_conf(self.fake_conf_file, k, v)
        # Test ctr ip, UserName and password unspecified
        tmp_dict = {'Storage/ControllerIP0': '10.10.10.1',
                    'Storage/ControllerIP1': '10.10.10.2',
                    'Storage/UserName': 'admin',
                    'Storage/UserPassword': '123456'}
        for k, v in tmp_dict.items():
            modify_conf(self.fake_conf_file, k, '')
            tmp_driver = HuaweiVolumeDriver(configuration=self.configuration)
            self.assertRaises(exception.InvalidInput,
                              tmp_driver.do_setup, None)
            modify_conf(self.fake_conf_file, k, v)
        # Test StoragePool unspecified
        modify_conf(self.fake_conf_file, 'LUN/StoragePool', '', attrib='Name')
        tmp_driver = HuaweiVolumeDriver(configuration=self.configuration)
        self.assertRaises(exception.InvalidInput,
                          tmp_driver.do_setup, None)
        modify_conf(self.fake_conf_file, 'LUN/StoragePool', 'RAID_001',
                    attrib='Name')
        # Test LUN type invalid
        modify_conf(self.fake_conf_file, 'LUN/LUNType', 'thick')
        tmp_driver = HuaweiVolumeDriver(configuration=self.configuration)
        tmp_driver.do_setup(None)
        self.assertRaises(exception.InvalidInput,
                          tmp_driver.create_volume, FAKE_VOLUME)
        modify_conf(self.fake_conf_file, 'LUN/LUNType', 'Thick')
        # Test OSType invalid
        modify_conf(self.fake_conf_file, 'Host', 'invalid_type',
                    attrib='OSType')
        tmp_driver = HuaweiVolumeDriver(configuration=self.configuration)
        self.assertRaises(exception.InvalidInput,
                          tmp_driver.do_setup, None)
        modify_conf(self.fake_conf_file, 'Host', 'Linux', attrib='OSType')
        # Test TargetIP not found
        modify_conf(self.fake_conf_file, 'iSCSI/DefaultTargetIP', '')
        modify_conf(self.fake_conf_file, 'iSCSI/Initiator', '', attrib='Name')
        tmp_driver = HuaweiVolumeDriver(configuration=self.configuration)
        tmp_driver.do_setup(None)
        tmp_driver.create_volume(FAKE_VOLUME)
        self.assertRaises(exception.InvalidInput,
                          tmp_driver.initialize_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        tmp_driver.delete_volume(FAKE_VOLUME)
        modify_conf(self.fake_conf_file, 'iSCSI/DefaultTargetIP',
                    '192.168.100.1')
        modify_conf(self.fake_conf_file, 'iSCSI/Initiator',
                    'iqn.1993-08.debian:01:ec2bff7ac3a3', attrib='Name')

    def test_volume_type(self):
        ctxt = context.get_admin_context()
        extra_specs = {'drivers:LUNType': 'Thin'}
        type_ref = volume_types.create(ctxt, 'THIN', extra_specs)
        FAKE_VOLUME['volume_type_id'] = type_ref['id']
        self.driver.create_volume(FAKE_VOLUME)
        self.assertEqual(LUN_INFO["ID"], VOLUME_SNAP_ID['vol'])
        self.assertEqual(LUN_INFO['Lun Type'], 'THIN')
        self.driver.delete_volume(FAKE_VOLUME)
        FAKE_VOLUME['volume_type_id'] = None

        # Test volume type invalid
        extra_specs = {'drivers:InvalidLUNType': 'Thin'}
        type_ref = volume_types.create(ctxt, 'Invalid_THIN', extra_specs)
        FAKE_VOLUME['volume_type_id'] = type_ref['id']
        self.driver.create_volume(FAKE_VOLUME)
        self.assertEqual(LUN_INFO["ID"], VOLUME_SNAP_ID['vol'])
        self.assertNotEqual(LUN_INFO['Lun Type'], 'THIN')
        self.driver.delete_volume(FAKE_VOLUME)
        FAKE_VOLUME['volume_type_id'] = None

    def test_create_delete_volume(self):
        # Test create lun cli exception
        set_error_flg('createlun')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume, FAKE_VOLUME)

        ret = self.driver.create_volume(FAKE_VOLUME)
        self.assertEqual(LUN_INFO['ID'], VOLUME_SNAP_ID['vol'])
        self.assertEqual(ret['provider_location'], LUN_INFO['ID'])

        # Test delete lun cli exception
        set_error_flg('dellun')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.delete_volume, FAKE_VOLUME)

        self.driver.delete_volume(FAKE_VOLUME)
        self.assertIsNone(LUN_INFO['ID'])
        self.assertIsNone(FAKE_VOLUME['provider_location'])

    def test_create_delete_cloned_volume(self):
        # Test no source volume
        self.assertRaises(exception.VolumeNotFound,
                          self.driver.create_cloned_volume,
                          FAKE_CLONED_VOLUME, FAKE_VOLUME)

        self.driver.create_volume(FAKE_VOLUME)
        # Test create luncopy failed
        self.assertEqual(LUN_INFO['ID'], VOLUME_SNAP_ID['vol'])
        set_error_flg('createluncopy')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          FAKE_CLONED_VOLUME, FAKE_VOLUME)
        self.assertEqual(CLONED_LUN_INFO['ID'], VOLUME_SNAP_ID['vol_copy'])
        self.driver.delete_volume(FAKE_CLONED_VOLUME)
        self.assertIsNone(CLONED_LUN_INFO['ID'])
        # Test start luncopy failed
        self.assertEqual(LUN_INFO['ID'], VOLUME_SNAP_ID['vol'])
        set_error_flg('chgluncopystatus')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          FAKE_CLONED_VOLUME, FAKE_VOLUME)
        self.assertIsNone(CLONED_LUN_INFO['ID'])
        self.assertEqual(LUN_INFO['ID'], VOLUME_SNAP_ID['vol'])
        # Test luncopy status abnormal
        LUNCOPY_SETTING['Status'] = 'Disable'
        self.assertEqual(LUN_INFO['ID'], '0')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          FAKE_CLONED_VOLUME, FAKE_VOLUME)
        self.assertIsNone(CLONED_LUN_INFO['ID'])
        self.assertEqual(LUN_INFO['ID'], VOLUME_SNAP_ID['vol'])
        LUNCOPY_SETTING['Status'] = 'Normal'
        # Test delete luncopy failed
        set_error_flg('delluncopy')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          FAKE_CLONED_VOLUME, FAKE_VOLUME)
        self.assertEqual(CLONED_LUN_INFO['ID'], VOLUME_SNAP_ID['vol_copy'])
        self.driver.delete_volume(FAKE_CLONED_VOLUME)
        self.assertIsNone(CLONED_LUN_INFO['ID'])
        # need to clean up LUNCopy
        LUNCOPY_INFO['Name'] = None
        LUNCOPY_INFO['ID'] = None
        LUNCOPY_INFO['Type'] = None
        LUNCOPY_INFO['State'] = None
        LUNCOPY_INFO['Status'] = None

        # Test normal create and delete cloned volume
        self.assertEqual(LUN_INFO['ID'], VOLUME_SNAP_ID['vol'])
        ret = self.driver.create_cloned_volume(FAKE_CLONED_VOLUME, FAKE_VOLUME)
        self.assertEqual(CLONED_LUN_INFO['ID'], VOLUME_SNAP_ID['vol_copy'])
        self.assertEqual(ret['provider_location'], CLONED_LUN_INFO['ID'])
        self.driver.delete_volume(FAKE_CLONED_VOLUME)
        self.assertIsNone(CLONED_LUN_INFO['ID'])
        self.assertIsNone(FAKE_CLONED_VOLUME['provider_location'])
        self.driver.delete_volume(FAKE_VOLUME)
        self.assertIsNone(LUN_INFO['ID'])

    def test_extend_volume(self):
        VOLUME_SIZE = 5
        # Test no extended volume
        self.assertRaises(exception.VolumeNotFound,
                          self.driver.extend_volume, FAKE_VOLUME, VOLUME_SIZE)

        self.driver.create_volume(FAKE_VOLUME)
        self.assertEqual(LUN_INFO['Size'], '2')
        # Test extend volume cli exception
        set_error_flg('addluntoextlun')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.extend_volume, FAKE_VOLUME, VOLUME_SIZE)
        self.assertEqual(CLONED_LUN_INFO['Name'], None)

        self.driver.extend_volume(FAKE_VOLUME, VOLUME_SIZE)
        self.assertEqual(LUN_INFO['Size'], VOLUME_SIZE)
        self.driver.delete_volume(FAKE_VOLUME)
        self.assertEqual(LUN_INFO['Name'], None)

    def test_create_delete_snapshot(self):
        # Test no resource pool
        RESPOOL_A_SIM['Valid Size'] = '0'
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_snapshot, FAKE_SNAPSHOT)
        RESPOOL_A_SIM['Valid Size'] = '5120'
        # Test no source volume
        self.assertRaises(exception.VolumeNotFound,
                          self.driver.create_snapshot, FAKE_SNAPSHOT)
        # Test create snapshot cli exception
        self.driver.create_volume(FAKE_VOLUME)
        set_error_flg('createsnapshot')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_snapshot,
                          FAKE_SNAPSHOT)
        self.assertEqual(LUN_INFO['ID'], VOLUME_SNAP_ID['vol'])
        # Test active snapshot failed
        set_error_flg('actvsnapshot')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_snapshot,
                          FAKE_SNAPSHOT)
        self.assertIsNone(SNAPSHOT_INFO['ID'])
        self.assertIsNone(SNAPSHOT_INFO['Status'])
        # Test disable snapshot failed
        set_error_flg('disablesnapshot')
        self.driver.create_snapshot(FAKE_SNAPSHOT)
        self.assertEqual(SNAPSHOT_INFO['ID'], VOLUME_SNAP_ID['snap'])
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.delete_snapshot,
                          FAKE_SNAPSHOT)
        self.assertEqual(SNAPSHOT_INFO['Status'], 'Active')
        # Test delsnapshot failed
        set_error_flg('delsnapshot')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.delete_snapshot,
                          FAKE_SNAPSHOT)
        self.assertEqual(SNAPSHOT_INFO['Status'], 'Disable')

        self.driver.delete_snapshot(FAKE_SNAPSHOT)

        # Test normal create and delete snapshot
        self.driver.create_volume(FAKE_VOLUME)
        ret = self.driver.create_snapshot(FAKE_SNAPSHOT)
        self.assertEqual(SNAPSHOT_INFO['ID'], VOLUME_SNAP_ID['snap'])
        self.assertEqual(SNAPSHOT_INFO['Status'], 'Active')
        self.assertEqual(ret['provider_location'], SNAPSHOT_INFO['ID'])
        self.driver.delete_snapshot(FAKE_SNAPSHOT)
        self.assertIsNone(SNAPSHOT_INFO['ID'])
        self.assertIsNone(SNAPSHOT_INFO['Status'])

    def test_create_delete_snapshot_volume(self):
        # Test no source snapshot
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          FAKE_CLONED_VOLUME, FAKE_SNAPSHOT)
        # Test normal create and delete snapshot volume
        self.driver.create_volume(FAKE_VOLUME)
        self.driver.create_snapshot(FAKE_SNAPSHOT)
        self.assertEqual(LUN_INFO['ID'], VOLUME_SNAP_ID['vol'])
        self.assertEqual(SNAPSHOT_INFO['ID'], VOLUME_SNAP_ID['snap'])
        ret = self.driver.create_volume_from_snapshot(FAKE_CLONED_VOLUME,
                                                      FAKE_SNAPSHOT)
        self.assertEqual(CLONED_LUN_INFO['ID'], VOLUME_SNAP_ID['vol_copy'])
        self.assertEqual(ret['provider_location'], CLONED_LUN_INFO['ID'])
        self.driver.delete_snapshot(FAKE_SNAPSHOT)
        self.driver.delete_volume(FAKE_VOLUME)
        self.driver.delete_volume(FAKE_CLONED_VOLUME)
        self.assertIsNone(LUN_INFO['ID'])
        self.assertIsNone(CLONED_LUN_INFO['ID'])
        self.assertIsNone(SNAPSHOT_INFO['ID'])

    def test_initialize_connection(self):
        # Test can not get iscsi iqn
        set_error_flg('showiscsitgtname')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.initialize_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test failed to get iSCSI port info
        set_error_flg('showiscsiip')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.initialize_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test create hostgroup failed
        set_error_flg('createhostgroup')
        MAP_INFO['Host Group ID'] = None
        MAP_INFO['Host Group Name'] = None
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.initialize_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test create host failed
        set_error_flg('addhost')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.initialize_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test add iSCSI initiator failed
        set_error_flg('addiscsiini')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.initialize_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test add hostport failed
        set_error_flg('addhostport')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.initialize_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test no volume
        FAKE_VOLUME['provider_location'] = '100'
        self.assertRaises(exception.VolumeNotFound,
                          self.driver.initialize_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        FAKE_VOLUME['provider_location'] = None
        # Test map volume failed
        self.driver.create_volume(FAKE_VOLUME)
        set_error_flg('addhostmap')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.initialize_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test normal initialize connection
        self.assertEqual(FAKE_VOLUME['provider_location'],
                         VOLUME_SNAP_ID['vol'])
        self.assertEqual(LUN_INFO['Owner Controller'], 'A')
        ret = self.driver.initialize_connection(FAKE_VOLUME, FAKE_CONNECTOR)
        iscsi_propers = ret['data']
        self.assertEqual(iscsi_propers['target_iqn'],
                         INITIATOR_SETTING['TargetIQN-form'])
        self.assertEqual(iscsi_propers['target_portal'],
                         INITIATOR_SETTING['Initiator TargetIP'] + ':3260')
        self.assertEqual(MAP_INFO["DEV LUN ID"], LUN_INFO['ID'])
        self.assertEqual(MAP_INFO["INI Port Info"],
                         FAKE_CONNECTOR['initiator'])
        self.assertEqual(LUN_INFO['Owner Controller'], 'B')
        self.driver.terminate_connection(FAKE_VOLUME, FAKE_CONNECTOR)
        self.driver.delete_volume(FAKE_VOLUME)
        self.assertIsNone(LUN_INFO['ID'])

    def test_terminate_connection(self):
        # Test no host was found
        self.assertRaises(exception.HostNotFound,
                          self.driver.terminate_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test no volume was found
        self.driver .create_volume(FAKE_VOLUME)
        self.driver.initialize_connection(FAKE_VOLUME, FAKE_CONNECTOR)
        FAKE_VOLUME['provider_location'] = None
        self.assertRaises(exception.VolumeNotFound,
                          self.driver.terminate_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        FAKE_VOLUME['provider_location'] = LUN_INFO['ID']
        # Test delete map failed
        set_error_flg('delhostmap')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.terminate_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Delete hostport failed
        set_error_flg('delhostport')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.terminate_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test delete initiator failed
        set_error_flg('deliscsiini')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.terminate_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test delete host failed
        set_error_flg('delhost')
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.terminate_connection,
                          FAKE_VOLUME, FAKE_CONNECTOR)
        # Test normal terminate connection
        self.assertEqual(LUN_INFO['ID'], VOLUME_SNAP_ID['vol'])
        self.driver.initialize_connection(FAKE_VOLUME, FAKE_CONNECTOR)
        self.driver.terminate_connection(FAKE_VOLUME, FAKE_CONNECTOR)
        self.assertIsNone(MAP_INFO["DEV LUN ID"])
        self.driver.delete_volume(FAKE_VOLUME)
        self.assertIsNone(LUN_INFO['ID'])

    def test_get_volume_stats(self):
        stats = self.driver.get_volume_stats(True)
        free_capacity = float(POOL_SETTING['Free Capacity']) / 1024
        self.assertEqual(stats['free_capacity_gb'], free_capacity)
        self.assertEqual(stats['storage_protocol'], 'iSCSI')


class HuaweiTFCDriverTestCase(test.TestCase):
    def __init__(self, *args, **kwargs):
        super(HuaweiTFCDriverTestCase, self).__init__(*args, **kwargs)

    def setUp(self):
        super(HuaweiTFCDriverTestCase, self).setUp()

        self.tmp_dir = tempfile.mkdtemp()
        self.fake_conf_file = self.tmp_dir + '/cinder_huawei_conf.xml'
        create_fake_conf_file(self.fake_conf_file)
        modify_conf(self.fake_conf_file, 'Storage/Protocol', 'FC')
        self.configuration = mox.MockObject(conf.Configuration)
        self.configuration.cinder_huawei_conf_file = self.fake_conf_file
        self.configuration.append_config_values(mox.IgnoreArg())

        self.stubs.Set(time, 'sleep', Fake_sleep)
        self.stubs.Set(utils, 'SSHPool', FakeSSHPool)
        self.stubs.Set(ssh_common.TseriesCommon, '_change_file_mode',
                       Fake_change_file_mode)
        self._init_driver()

    def _init_driver(self):
        Curr_test[0] = 'T'
        self.driver = HuaweiVolumeDriver(configuration=self.configuration)
        self.driver.do_setup(None)

    def tearDown(self):
        if os.path.exists(self.fake_conf_file):
            os.remove(self.fake_conf_file)
        shutil.rmtree(self.tmp_dir)
        super(HuaweiTFCDriverTestCase, self).tearDown()

    def test_validate_connector_failed(self):
        invalid_connector = {'host': 'testhost'}
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.validate_connector,
                          invalid_connector)

    def test_create_delete_volume(self):
        self.driver.create_volume(FAKE_VOLUME)
        self.assertEqual(LUN_INFO['ID'], VOLUME_SNAP_ID['vol'])
        self.driver.delete_volume(FAKE_VOLUME)
        self.assertIsNone(LUN_INFO['ID'])

    def test_create_delete_snapshot(self):
        self.driver.create_volume(FAKE_VOLUME)
        self.driver.create_snapshot(FAKE_SNAPSHOT)
        self.assertEqual(SNAPSHOT_INFO['ID'], VOLUME_SNAP_ID['snap'])
        self.driver.delete_snapshot(FAKE_SNAPSHOT)
        self.assertIsNone(SNAPSHOT_INFO['ID'])
        self.driver.delete_volume(FAKE_VOLUME)
        self.assertIsNone(LUN_INFO['ID'])

    def test_create_cloned_volume(self):
        self.driver.create_volume(FAKE_VOLUME)
        ret = self.driver.create_cloned_volume(FAKE_CLONED_VOLUME, FAKE_VOLUME)
        self.assertEqual(CLONED_LUN_INFO['ID'], VOLUME_SNAP_ID['vol_copy'])
        self.assertEqual(ret['provider_location'], CLONED_LUN_INFO['ID'])
        self.driver.delete_volume(FAKE_CLONED_VOLUME)
        self.driver.delete_volume(FAKE_VOLUME)
        self.assertIsNone(CLONED_LUN_INFO['ID'])
        self.assertIsNone(LUN_INFO['ID'])

    def test_create_snapshot_volume(self):
        self.driver.create_volume(FAKE_VOLUME)
        self.driver.create_snapshot(FAKE_SNAPSHOT)
        ret = self.driver.create_volume_from_snapshot(FAKE_CLONED_VOLUME,
                                                      FAKE_SNAPSHOT)
        self.assertEqual(CLONED_LUN_INFO['ID'], VOLUME_SNAP_ID['vol_copy'])
        self.assertEqual(ret['provider_location'], CLONED_LUN_INFO['ID'])
        self.driver.delete_volume(FAKE_CLONED_VOLUME)
        self.driver.delete_volume(FAKE_VOLUME)
        self.assertIsNone(CLONED_LUN_INFO['ID'])
        self.assertIsNone(LUN_INFO['ID'])

    def test_initialize_terminitat_connection(self):
        self.driver.create_volume(FAKE_VOLUME)
        ret = self.driver.initialize_connection(FAKE_VOLUME, FAKE_CONNECTOR)
        fc_properties = ret['data']
        self.assertEqual(fc_properties['target_wwn'],
                         INITIATOR_SETTING['WWN'])
        self.assertEqual(MAP_INFO["DEV LUN ID"], LUN_INFO['ID'])

        self.driver.terminate_connection(FAKE_VOLUME, FAKE_CONNECTOR)
        self.assertIsNone(MAP_INFO["DEV LUN ID"])
        self.assertIsNone(MAP_INFO["Host LUN ID"])
        self.driver.delete_volume(FAKE_VOLUME)
        self.assertIsNone(LUN_INFO['ID'])

    def _test_get_volume_stats(self):
        stats = self.driver.get_volume_stats(True)
        fakecapacity = float(POOL_SETTING['Free Capacity']) / 1024
        self.assertEqual(stats['free_capacity_gb'], fakecapacity)
        self.assertEqual(stats['storage_protocol'], 'FC')


class HuaweiDorado5100FCDriverTestCase(HuaweiTFCDriverTestCase):
    def __init__(self, *args, **kwargs):
        super(HuaweiDorado5100FCDriverTestCase, self).__init__(*args, **kwargs)

    def setUp(self):
        super(HuaweiDorado5100FCDriverTestCase, self).setUp()

    def _init_driver(self):
        Curr_test[0] = 'Dorado5100'
        modify_conf(self.fake_conf_file, 'Storage/Product', 'Dorado')
        self.driver = HuaweiVolumeDriver(configuration=self.configuration)
        self.driver.do_setup(None)

    def test_create_cloned_volume(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          FAKE_CLONED_VOLUME, FAKE_VOLUME)

    def test_create_snapshot_volume(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          FAKE_CLONED_VOLUME, FAKE_SNAPSHOT)


class HuaweiDorado2100G2FCDriverTestCase(HuaweiTFCDriverTestCase):
    def __init__(self, *args, **kwargs):
        super(HuaweiDorado2100G2FCDriverTestCase, self).__init__(*args,
                                                                 **kwargs)

    def setUp(self):
        super(HuaweiDorado2100G2FCDriverTestCase, self).setUp()

    def _init_driver(self):
        Curr_test[0] = 'Dorado2100G2'
        modify_conf(self.fake_conf_file, 'Storage/Product', 'Dorado')
        self.driver = HuaweiVolumeDriver(configuration=self.configuration)
        self.driver.do_setup(None)

    def test_create_cloned_volume(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          FAKE_CLONED_VOLUME, FAKE_VOLUME)

    def test_create_delete_snapshot(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_snapshot, FAKE_SNAPSHOT)

    def test_create_snapshot_volume(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          FAKE_CLONED_VOLUME, FAKE_SNAPSHOT)

    def test_extend_volume(self):
        NEWSIZE = 5
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.extend_volume,
                          FAKE_VOLUME, NEWSIZE)


class HuaweiDorado5100ISCSIDriverTestCase(HuaweiTISCSIDriverTestCase):
    def __init__(self, *args, **kwargs):
        super(HuaweiDorado5100ISCSIDriverTestCase, self).__init__(*args,
                                                                  **kwargs)

    def setUp(self):
        super(HuaweiDorado5100ISCSIDriverTestCase, self).setUp()

    def _init_driver(self):
        Curr_test[0] = 'Dorado5100'
        modify_conf(self.fake_conf_file, 'Storage/Product', 'Dorado')
        self.driver = HuaweiVolumeDriver(configuration=self.configuration)
        self.driver.do_setup(None)

    def test_create_delete_cloned_volume(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          FAKE_CLONED_VOLUME, FAKE_VOLUME)

    def test_create_delete_snapshot_volume(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          FAKE_CLONED_VOLUME, FAKE_SNAPSHOT)

    def test_volume_type(self):
        pass


class HuaweiDorado2100G2ISCSIDriverTestCase(HuaweiTISCSIDriverTestCase):
    def __init__(self, *args, **kwargs):
        super(HuaweiDorado2100G2ISCSIDriverTestCase, self).__init__(*args,
                                                                    **kwargs)

    def setUp(self):
        super(HuaweiDorado2100G2ISCSIDriverTestCase, self).setUp()

    def _init_driver(self):
        Curr_test[0] = 'Dorado2100G2'
        modify_conf(self.fake_conf_file, 'Storage/Product', 'Dorado')
        self.driver = HuaweiVolumeDriver(configuration=self.configuration)
        self.driver.do_setup(None)

    def test_conf_invalid(self):
        pass

    def test_create_delete_cloned_volume(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          FAKE_CLONED_VOLUME, FAKE_VOLUME)

    def test_create_delete_snapshot(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_snapshot, FAKE_SNAPSHOT)

    def test_create_delete_snapshot_volume(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          FAKE_CLONED_VOLUME, FAKE_SNAPSHOT)

    def test_initialize_connection(self):
        self.driver.create_volume(FAKE_VOLUME)
        ret = self.driver.initialize_connection(FAKE_VOLUME, FAKE_CONNECTOR)
        iscsi_propers = ret['data']
        self.assertEqual(iscsi_propers['target_iqn'],
                         INITIATOR_SETTING['TargetIQN-form'])
        self.assertEqual(iscsi_propers['target_portal'],
                         INITIATOR_SETTING['Initiator TargetIP'] + ':3260')
        self.assertEqual(MAP_INFO["DEV LUN ID"], LUN_INFO['ID'])
        self.assertEqual(MAP_INFO["INI Port Info"],
                         FAKE_CONNECTOR['initiator'])
        self.driver.terminate_connection(FAKE_VOLUME, FAKE_CONNECTOR)
        self.driver.delete_volume(FAKE_VOLUME)
        self.assertIsNone(LUN_INFO['ID'])

    def test_extend_volume(self):
        NEWSIZE = 5
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.extend_volume,
                          FAKE_VOLUME, NEWSIZE)


class SSHMethodTestCase(test.TestCase):
    def __init__(self, *args, **kwargs):
        super(SSHMethodTestCase, self).__init__(*args, **kwargs)

    def setUp(self):
        super(SSHMethodTestCase, self).setUp()

        self.tmp_dir = tempfile.mkdtemp()
        self.fake_conf_file = self.tmp_dir + '/cinder_huawei_conf.xml'
        create_fake_conf_file(self.fake_conf_file)
        self.configuration = mox.MockObject(conf.Configuration)
        self.configuration.cinder_huawei_conf_file = self.fake_conf_file
        self.configuration.append_config_values(mox.IgnoreArg())

        self.stubs.Set(time, 'sleep', Fake_sleep)
        self.stubs.Set(utils, 'SSHPool', FakeSSHPool)
        self.stubs.Set(ssh_common.TseriesCommon, '_change_file_mode',
                       Fake_change_file_mode)
        Curr_test[0] = 'T'
        self.driver = HuaweiVolumeDriver(configuration=self.configuration)
        self.driver.do_setup(None)

    def tearDown(self):
        if os.path.exists(self.fake_conf_file):
            os.remove(self.fake_conf_file)
        shutil.rmtree(self.tmp_dir)
        super(SSHMethodTestCase, self).tearDown()

    def test_reach_max_connection_limit(self):
        self.stubs.Set(FakeChannel, 'recv', self._fake_recv1)
        self.assertRaises(exception.CinderException,
                          self.driver.create_volume, FAKE_VOLUME)

    def test_socket_timeout(self):
        self.stubs.Set(FakeChannel, 'recv', self._fake_recv2)
        self.assertRaises(socket.timeout,
                          self.driver.create_volume, FAKE_VOLUME)

    def _fake_recv1(self, nbytes):
        return "No response message"

    def _fake_recv2(self, nBytes):
        raise socket.timeout()


class HuaweiUtilsTestCase(test.TestCase):
    def __init__(self, *args, **kwargs):
        super(HuaweiUtilsTestCase, self).__init__(*args, **kwargs)

    def setUp(self):
        super(HuaweiUtilsTestCase, self).setUp()

        self.tmp_dir = tempfile.mkdtemp()
        self.fake_conf_file = self.tmp_dir + '/cinder_huawei_conf.xml'
        create_fake_conf_file(self.fake_conf_file)

    def tearDown(self):
        if os.path.exists(self.fake_conf_file):
            os.remove(self.fake_conf_file)
        shutil.rmtree(self.tmp_dir)
        super(HuaweiUtilsTestCase, self).tearDown()

    def test_parse_xml_file_ioerror(self):
        tmp_fonf_file = '/xxx/cinder_huawei_conf.xml'
        self.assertRaises(IOError, huawei_utils.parse_xml_file, tmp_fonf_file)

    def test_is_xml_item_exist(self):
        root = huawei_utils.parse_xml_file(self.fake_conf_file)
        res = huawei_utils.is_xml_item_exist(root, 'Storage/UserName')
        self.assertTrue(res)
        res = huawei_utils.is_xml_item_exist(root, 'xxx')
        self.assertFalse(res)
        res = huawei_utils.is_xml_item_exist(root, 'LUN/StoragePool', 'Name')
        self.assertTrue(res)
        res = huawei_utils.is_xml_item_exist(root, 'LUN/StoragePool', 'xxx')
        self.assertFalse(res)

    def test_is_xml_item_valid(self):
        root = huawei_utils.parse_xml_file(self.fake_conf_file)
        res = huawei_utils.is_xml_item_valid(root, 'LUN/LUNType',
                                             ['Thin', 'Thick'])
        self.assertTrue(res)
        res = huawei_utils.is_xml_item_valid(root, 'LUN/LUNType', ['test'])
        self.assertFalse(res)
        res = huawei_utils.is_xml_item_valid(root, 'Host',
                                             ['Linux', 'Windows'], 'OSType')
        self.assertTrue(res)
        res = huawei_utils.is_xml_item_valid(root, 'Host', ['test'], 'OSType')
        self.assertFalse(res)

    def test_get_conf_host_os_type(self):
        # Default os is Linux
        res = huawei_utils.get_conf_host_os_type('10.10.10.1',
                                                 self.fake_conf_file)
        self.assertEqual(res, '0')
        modify_conf(self.fake_conf_file, 'Host', 'Windows', 'OSType')
        res = huawei_utils.get_conf_host_os_type(FAKE_CONNECTOR['ip'],
                                                 self.fake_conf_file)
        self.assertEqual(res, '1')



# Copyright (c) 2012 - 2014 EMC Corporation, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import os
import shutil
import tempfile
import time
from xml.dom.minidom import Document

import mock

from cinder import exception
from cinder.openstack.common import log as logging
from cinder import test
from cinder import units
from cinder.volume.drivers.emc.emc_smis_common import EMCSMISCommon
from cinder.volume.drivers.emc.emc_smis_fc import EMCSMISFCDriver
from cinder.volume.drivers.emc.emc_smis_iscsi import EMCSMISISCSIDriver
from cinder.volume import volume_types

CINDER_EMC_CONFIG_FILE = '/etc/cinder/cinder_emc_config.xml'
LOG = logging.getLogger(__name__)


class EMC_StorageVolume(dict):
    pass


class SE_ConcreteJob(dict):
    pass


class SE_StorageHardwareID(dict):
    pass


class FakeCIMInstanceName(dict):

    def fake_getinstancename(self, classname, bindings):
        instancename = FakeCIMInstanceName()
        for key in bindings:
            instancename[key] = bindings[key]
        instancename.classname = classname
        instancename.namespace = 'root/emc'
        return instancename


class FakeDB():
    def volume_update(self, context, volume_id, model_update):
        pass

    def snapshot_update(self, context, snapshot_id, model_update):
        pass

    def volume_get(self, context, volume_id):
        conn = FakeEcomConnection()
        objectpath = {}
        objectpath['CreationClassName'] = 'Clar_StorageVolume'
        if volume_id == 'vol1':
            device_id = '1'
            objectpath['DeviceID'] = device_id
        else:
            objectpath['DeviceID'] = volume_id
        return conn.GetInstance(objectpath)


class EMCSMISCommonData():
    connector = {'ip': '10.0.0.2',
                 'initiator': 'iqn.1993-08.org.debian:01:222',
                 'wwpns': ["123456789012345", "123456789054321"],
                 'wwnns': ["223456789012345", "223456789054321"],
                 'host': 'fakehost'}

    config_file_name = 'cinder_emc_config.xml'
    storage_system = 'CLARiiON+APM00123456789'
    storage_system_vmax = 'SYMMETRIX+000195900551'
    lunmaskctrl_id =\
        'CLARiiON+APM00123456789+00aa11bb22cc33dd44ff55gg66hh77ii88jj'
    initiator1 = 'iqn.1993-08.org.debian:01:1a2b3c4d5f6g'
    stconf_service_creationclass = 'Clar_StorageConfigurationService'
    ctrlconf_service_creationclass = 'Clar_ControllerConfigurationService'
    rep_service_creationclass = 'Clar_ReplicationService'
    vol_creationclass = 'Clar_StorageVolume'
    pool_creationclass = 'Clar_UnifiedStoragePool'
    lunmask_creationclass = 'Clar_LunMaskingSCSIProtocolController'
    unit_creationclass = 'CIM_ProtocolControllerForUnit'
    storage_type = 'gold'

    test_volume = {'name': 'vol1',
                   'size': 1,
                   'volume_name': 'vol1',
                   'id': '1',
                   'provider_auth': None,
                   'project_id': 'project',
                   'display_name': 'vol1',
                   'display_description': 'test volume',
                   'volume_type_id': None}
    test_failed_volume = {'name': 'failed_vol',
                          'size': 1,
                          'volume_name': 'failed_vol',
                          'id': '4',
                          'provider_auth': None,
                          'project_id': 'project',
                          'display_name': 'failed_vol',
                          'display_description': 'test failed volume',
                          'volume_type_id': None}
    test_snapshot = {'name': 'snapshot1',
                     'size': 1,
                     'id': '4444',
                     'volume_name': 'vol-vol1',
                     'volume_size': 1,
                     'project_id': 'project'}
    test_clone = {'name': 'clone1',
                  'size': 1,
                  'volume_name': 'vol1',
                  'id': '2',
                  'provider_auth': None,
                  'project_id': 'project',
                  'display_name': 'clone1',
                  'display_description': 'volume created from snapshot',
                  'volume_type_id': None}
    test_clone3 = {'name': 'clone3',
                   'size': 1,
                   'volume_name': 'vol1',
                   'id': '3',
                   'provider_auth': None,
                   'project_id': 'project',
                   'display_name': 'clone3',
                   'display_description': 'cloned volume',
                   'volume_type_id': None}
    test_snapshot_vmax = {'name': 'snapshot_vmax',
                          'size': 1,
                          'id': '4445',
                          'volume_name': 'vol-vol1',
                          'volume_size': 1,
                          'project_id': 'project'}
    failed_snapshot_replica = {'name': 'failed_snapshot_replica',
                               'size': 1,
                               'volume_name': 'vol-vol1',
                               'id': '5',
                               'provider_auth': None,
                               'project_id': 'project',
                               'display_name': 'vol1',
                               'display_description':
                               'failed snapshot replica',
                               'volume_type_id': None}
    failed_snapshot_sync = {'name': 'failed_snapshot_sync',
                            'size': 1,
                            'volume_name': 'vol-vol1',
                            'id': '6',
                            'provider_auth': None,
                            'project_id': 'project',
                            'display_name': 'failed_snapshot_sync',
                            'display_description': 'failed snapshot sync',
                            'volume_type_id': None}
    failed_clone_replica = {'name': 'failed_clone_replica',
                            'size': 1,
                            'volume_name': 'vol1',
                            'id': '7',
                            'provider_auth': None,
                            'project_id': 'project',
                            'display_name': 'vol1',
                            'display_description': 'failed clone replica',
                            'volume_type_id': None}
    failed_clone_sync = {'name': 'failed_clone_sync',
                         'size': 1,
                         'volume_name': 'vol1',
                         'id': '8',
                         'provider_auth': None,
                         'project_id': 'project',
                         'display_name': 'vol1',
                         'display_description': 'failed clone sync',
                         'volume_type_id': None}
    failed_delete_vol = {'name': 'failed_delete_vol',
                         'size': 1,
                         'volume_name': 'failed_delete_vol',
                         'id': '99999',
                         'provider_auth': None,
                         'project_id': 'project',
                         'display_name': 'failed delete vol',
                         'display_description': 'failed delete volume',
                         'volume_type_id': None}
    failed_extend_vol = {'name': 'failed_extend_vol',
                         'size': 1,
                         'volume_name': 'failed_extend_vol',
                         'id': '9',
                         'provider_auth': None,
                         'project_id': 'project',
                         'display_name': 'failed_extend_vol',
                         'display_description': 'test failed extend volume',
                         'volume_type_id': None}


class FakeEcomConnection():

    def __init__(self, *args, **kwargs):
        self.data = EMCSMISCommonData()

    def InvokeMethod(self, MethodName, Service, ElementName=None, InPool=None,
                     ElementType=None, Size=None,
                     SyncType=None, SourceElement=None,
                     Operation=None, Synchronization=None,
                     TheElements=None, TheElement=None,
                     LUNames=None, InitiatorPortIDs=None, DeviceAccesses=None,
                     ProtocolControllers=None,
                     MaskingGroup=None, Members=None,
                     HardwareId=None):

        rc = 0L
        myjob = SE_ConcreteJob()
        myjob.classname = 'SE_ConcreteJob'
        myjob['InstanceID'] = '9999'
        myjob['status'] = 'success'
        myjob['type'] = ElementName
        if ElementName == 'failed_vol' and \
                MethodName == 'CreateOrModifyElementFromStoragePool':
            rc = 10L
            myjob['status'] = 'failure'
        elif TheElement and TheElement['ElementName'] == 'failed_extend_vol' \
                and MethodName == 'CreateOrModifyElementFromStoragePool':
            rc = 10L
            myjob['status'] = 'failure'
        elif MethodName == 'CreateOrModifyElementFromStoragePool':
            rc = 0L
            myjob['status'] = 'success'
        elif ElementName == 'failed_snapshot_replica' and \
                MethodName == 'CreateElementReplica':
            rc = 10L
            myjob['status'] = 'failure'
        elif Synchronization and \
                Synchronization['SyncedElement']['ElementName'] \
                == 'failed_snapshot_sync' and \
                MethodName == 'ModifyReplicaSynchronization':
            rc = 10L
            myjob['status'] = 'failure'
        elif ElementName == 'failed_clone_replica' and \
                MethodName == 'CreateElementReplica':
            rc = 10L
            myjob['status'] = 'failure'
        elif Synchronization and \
                Synchronization['SyncedElement']['ElementName'] \
                == 'failed_clone_sync' and \
                MethodName == 'ModifyReplicaSynchronization':
            rc = 10L
            myjob['status'] = 'failure'
        elif TheElements and \
                TheElements[0]['DeviceID'] == '99999' and \
                MethodName == 'EMCReturnToStoragePool':
            rc = 10L
            myjob['status'] = 'failure'
        elif HardwareId:
            rc = 0L
            targetendpoints = {}
            endpoints = []
            endpoint = {}
            endpoint['Name'] = '1234567890123'
            endpoints.append(endpoint)
            endpoint2 = {}
            endpoint2['Name'] = '0987654321321'
            endpoints.append(endpoint2)
            targetendpoints['TargetEndpoints'] = endpoints
            return rc, targetendpoints

        job = {'Job': myjob}
        return rc, job

    def EnumerateInstanceNames(self, name):
        result = None
        if name == 'EMC_ReplicationService':
            result = self._enum_replicationservices()
        elif name == 'EMC_StorageConfigurationService':
            result = self._enum_stconfsvcs()
        elif name == 'EMC_ControllerConfigurationService':
            result = self._enum_ctrlconfsvcs()
        elif name == 'EMC_VirtualProvisioningPool':
            result = self._enum_pools()
        elif name == 'EMC_UnifiedStoragePool':
            result = self._enum_pools()
        elif name == 'EMC_StorageVolume':
            result = self._enum_storagevolumes()
        elif name == 'Clar_StorageVolume':
            result = self._enum_storagevolumes()
        elif name == 'SE_StorageSynchronized_SV_SV':
            result = self._enum_syncsvsvs()
        elif name == 'CIM_ProtocolControllerForUnit':
            result = self._enum_unitnames()
        elif name == 'EMC_LunMaskingSCSIProtocolController':
            result = self._enum_lunmaskctrls()
        elif name == 'EMC_StorageProcessorSystem':
            result = self._enum_processors()
        elif name == 'EMC_StorageHardwareIDManagementService':
            result = self._enum_hdwidmgmts()
        else:
            result = self._default_enum()
        return result

    def EnumerateInstances(self, name):
        result = None
        if name == 'EMC_VirtualProvisioningPool':
            result = self._enum_pool_details()
        elif name == 'EMC_UnifiedStoragePool':
            result = self._enum_pool_details()
        elif name == 'SE_StorageHardwareID':
            result = self._enum_storhdwids()
        else:
            result = self._default_enum()
        return result

    def GetInstance(self, objectpath, LocalOnly=False):
        try:
            name = objectpath['CreationClassName']
        except KeyError:
            name = objectpath.classname
        result = None
        if name == 'Clar_StorageVolume' or name == 'Symm_StorageVolume':
            result = self._getinstance_storagevolume(objectpath)
        elif name == 'CIM_ProtocolControllerForUnit':
            result = self._getinstance_unit(objectpath)
        elif name == 'Clar_LunMaskingSCSIProtocolController':
            result = self._getinstance_lunmask()
        elif name == 'SE_ConcreteJob':
            result = self._getinstance_job(objectpath)
        elif name == 'SE_StorageSynchronized_SV_SV':
            result = self._getinstance_syncsvsv(objectpath)
        else:
            result = self._default_getinstance(objectpath)
        return result

    def Associators(self, objectpath, resultClass='EMC_StorageHardwareID'):
        result = None
        if resultClass == 'EMC_StorageHardwareID':
            result = self._assoc_hdwid()
        elif resultClass == 'EMC_iSCSIProtocolEndpoint':
            result = self._assoc_endpoint()
        # Added test for EMC_StorageVolume
        elif resultClass == 'EMC_StorageVolume':
            result = self._assoc_storagevolume(objectpath)
        else:
            result = self._default_assoc(objectpath)
        return result

    def AssociatorNames(self, objectpath,
                        resultClass='EMC_LunMaskingSCSIProtocolController'):
        result = None
        if resultClass == 'EMC_LunMaskingSCSIProtocolController':
            result = self._assocnames_lunmaskctrl()
        else:
            result = self._default_assocnames(objectpath)
        return result

    def ReferenceNames(self, objectpath,
                       ResultClass='CIM_ProtocolControllerForUnit'):
        result = None
        if ResultClass == 'CIM_ProtocolControllerForUnit':
            result = self._ref_unitnames()
        else:
            result = self._default_ref(objectpath)
        return result

    def _ref_unitnames(self):
        unitnames = []
        unitname = {}

        dependent = {}
        dependent['CreationClassName'] = self.data.vol_creationclass
        dependent['DeviceID'] = self.data.test_volume['id']
        dependent['ElementName'] = self.data.test_volume['name']
        dependent['SystemName'] = self.data.storage_system

        antecedent = {}
        antecedent['CreationClassName'] = self.data.lunmask_creationclass
        antecedent['DeviceID'] = self.data.lunmaskctrl_id
        antecedent['SystemName'] = self.data.storage_system

        unitname['Dependent'] = dependent
        unitname['Antecedent'] = antecedent
        unitname['CreationClassName'] = self.data.unit_creationclass
        unitnames.append(unitname)

        return unitnames

    def _default_ref(self, objectpath):
        return objectpath

    def _assoc_hdwid(self):
        assocs = []
        assoc = {}
        assoc['StorageID'] = self.data.connector['initiator']
        assocs.append(assoc)
        for wwpn in self.data.connector['wwpns']:
            assoc2 = {}
            assoc2['StorageID'] = wwpn
            assocs.append(assoc2)
        return assocs

    def _assoc_endpoint(self):
        assocs = []
        assoc = {}
        assoc['Name'] = 'iqn.1992-04.com.emc:cx.apm00123907237.a8,t,0x0001'
        assoc['SystemName'] = self.data.storage_system + '+SP_A+8'
        assocs.append(assoc)
        return assocs

    # Added test for EMC_StorageVolume associators
    def _assoc_storagevolume(self, objectpath):
        assocs = []
        if objectpath['type'] == 'failed_delete_vol':
            vol = self.data.failed_delete_vol
        elif objectpath['type'] == 'vol1':
            vol = self.data.test_volume
        elif objectpath['type'] == 'failed_vol':
            vol = self.data.test_failed_volume
        elif objectpath['type'] == 'failed_clone_sync':
            vol = self.data.failed_clone_sync
        elif objectpath['type'] == 'failed_clone_replica':
            vol = self.data.failed_clone_replica
        elif objectpath['type'] == 'failed_snapshot_replica':
            vol = self.data.failed_snapshot_replica
        elif objectpath['type'] == 'failed_snapshot_sync':
            vol = self.data.failed_snapshot_sync
        elif objectpath['type'] == 'clone1':
            vol = self.data.test_clone
        elif objectpath['type'] == 'clone3':
            vol = self.data.test_clone3
        elif objectpath['type'] == 'snapshot1':
            vol = self.data.test_snapshot
        elif objectpath['type'] == 'snapshot_vmax':
            vol = self.data.test_snapshot_vmax
        elif objectpath['type'] == 'failed_extend_vol':
            vol = self.data.failed_extend_vol
        else:
            return None

        vol['DeviceID'] = vol['id']
        assoc = self._getinstance_storagevolume(vol)
        assocs.append(assoc)
        return assocs

    def _default_assoc(self, objectpath):
        return objectpath

    def _assocnames_lunmaskctrl(self):
        return self._enum_lunmaskctrls()

    def _default_assocnames(self, objectpath):
        return objectpath

    def _getinstance_storagevolume(self, objectpath):
        foundinstance = None
        instance = EMC_StorageVolume()
        vols = self._enum_storagevolumes()
        for vol in vols:
            if vol['DeviceID'] == objectpath['DeviceID']:
                instance = vol
                break
        if not instance:
            foundinstance = None
        else:
            foundinstance = instance
        return foundinstance

    def _getinstance_syncsvsv(self, objectpath):
        foundsync = None
        syncs = self._enum_syncsvsvs()
        for sync in syncs:
            if (sync['SyncedElement'] == objectpath['SyncedElement'] and
                    sync['SystemElement'] == objectpath['SystemElement']):
                foundsync = sync
                break
        return foundsync

    def _getinstance_lunmask(self):
        lunmask = {}
        lunmask['CreationClassName'] = self.data.lunmask_creationclass
        lunmask['DeviceID'] = self.data.lunmaskctrl_id
        lunmask['SystemName'] = self.data.storage_system
        return lunmask

    def _getinstance_unit(self, objectpath):
        unit = {}

        dependent = {}
        dependent['CreationClassName'] = self.data.vol_creationclass
        dependent['DeviceID'] = self.data.test_volume['id']
        dependent['ElementName'] = self.data.test_volume['name']
        dependent['SystemName'] = self.data.storage_system

        antecedent = {}
        antecedent['CreationClassName'] = self.data.lunmask_creationclass
        antecedent['DeviceID'] = self.data.lunmaskctrl_id
        antecedent['SystemName'] = self.data.storage_system

        unit['Dependent'] = dependent
        unit['Antecedent'] = antecedent
        unit['CreationClassName'] = self.data.unit_creationclass
        unit['DeviceNumber'] = '0'

        return unit

    def _getinstance_job(self, jobpath):
        jobinstance = {}
        jobinstance['InstanceID'] = '9999'
        if jobpath['status'] == 'failure':
            jobinstance['JobState'] = 10
            jobinstance['ErrorCode'] = 99
            jobinstance['ErrorDescription'] = 'Failure'
        else:
            jobinstance['JobState'] = 7
            jobinstance['ErrorCode'] = 0
            jobinstance['ErrorDescription'] = ''
        return jobinstance

    def _default_getinstance(self, objectpath):
        return objectpath

    def _enum_replicationservices(self):
        rep_services = []
        rep_service = {}
        rep_service['SystemName'] = self.data.storage_system
        rep_service['CreationClassName'] = self.data.rep_service_creationclass
        rep_services.append(rep_service)
        return rep_services

    def _enum_stconfsvcs(self):
        conf_services = []
        conf_service = {}
        conf_service['SystemName'] = self.data.storage_system
        conf_service['CreationClassName'] =\
            self.data.stconf_service_creationclass
        conf_services.append(conf_service)
        return conf_services

    def _enum_ctrlconfsvcs(self):
        conf_services = []
        conf_service = {}
        conf_service['SystemName'] = self.data.storage_system
        conf_service['CreationClassName'] =\
            self.data.ctrlconf_service_creationclass
        conf_services.append(conf_service)
        return conf_services

    def _enum_pools(self):
        pools = []
        pool = {}
        pool['InstanceID'] = self.data.storage_system + '+U+' +\
            self.data.storage_type
        pool['CreationClassName'] = 'Clar_UnifiedStoragePool'
        pools.append(pool)
        return pools

    def _enum_pool_details(self):
        pools = []
        pool = {}
        pool['InstanceID'] = self.data.storage_system + '+U+' +\
            self.data.storage_type
        pool['CreationClassName'] = 'Clar_UnifiedStoragePool'
        pool['TotalManagedSpace'] = 12345678
        pool['RemainingManagedSpace'] = 123456
        pools.append(pool)
        return pools

    def _enum_storagevolumes(self):
        vols = []

        vol = EMC_StorageVolume()
        vol['name'] = self.data.test_volume['name']
        vol['CreationClassName'] = 'Clar_StorageVolume'
        vol['ElementName'] = self.data.test_volume['name']
        vol['DeviceID'] = self.data.test_volume['id']
        vol['SystemName'] = self.data.storage_system
        # Added vol to vol.path
        vol['SystemCreationClassName'] = 'Clar_StorageSystem'
        vol.path = vol
        vol.path.classname = vol['CreationClassName']

        name = {}
        name['classname'] = 'Clar_StorageVolume'
        keys = {}
        keys['CreationClassName'] = 'Clar_StorageVolume'
        keys['SystemName'] = self.data.storage_system
        keys['DeviceID'] = vol['DeviceID']
        keys['SystemCreationClassName'] = 'Clar_StorageSystem'
        name['keybindings'] = keys
        vol['provider_location'] = str(name)

        vols.append(vol)

        snap_vol = EMC_StorageVolume()
        snap_vol['name'] = self.data.test_snapshot['name']
        snap_vol['CreationClassName'] = 'Clar_StorageVolume'
        snap_vol['ElementName'] = self.data.test_snapshot['name']
        snap_vol['DeviceID'] = self.data.test_snapshot['id']
        snap_vol['SystemName'] = self.data.storage_system
        # Added vol to path
        snap_vol['SystemCreationClassName'] = 'Clar_StorageSystem'
        snap_vol.path = snap_vol
        snap_vol.path.classname = snap_vol['CreationClassName']

        name2 = {}
        name2['classname'] = 'Clar_StorageVolume'
        keys2 = {}
        keys2['CreationClassName'] = 'Clar_StorageVolume'
        keys2['SystemName'] = self.data.storage_system
        keys2['DeviceID'] = snap_vol['DeviceID']
        keys2['SystemCreationClassName'] = 'Clar_StorageSystem'
        name2['keybindings'] = keys2
        snap_vol['provider_location'] = str(name2)

        vols.append(snap_vol)

        clone_vol = EMC_StorageVolume()
        clone_vol['name'] = self.data.test_clone['name']
        clone_vol['CreationClassName'] = 'Clar_StorageVolume'
        clone_vol['ElementName'] = self.data.test_clone['name']
        clone_vol['DeviceID'] = self.data.test_clone['id']
        clone_vol['SystemName'] = self.data.storage_system
        # Added vol to vol.path
        clone_vol['SystemCreationClassName'] = 'Clar_StorageSystem'
        clone_vol.path = clone_vol
        clone_vol.path.classname = clone_vol['CreationClassName']
        vols.append(clone_vol)

        clone_vol3 = EMC_StorageVolume()
        clone_vol3['name'] = self.data.test_clone3['name']
        clone_vol3['CreationClassName'] = 'Clar_StorageVolume'
        clone_vol3['ElementName'] = self.data.test_clone3['name']
        clone_vol3['DeviceID'] = self.data.test_clone3['id']
        clone_vol3['SystemName'] = self.data.storage_system
        # Added vol to vol.path
        clone_vol3['SystemCreationClassName'] = 'Clar_StorageSystem'
        clone_vol3.path = clone_vol3
        clone_vol3.path.classname = clone_vol3['CreationClassName']
        vols.append(clone_vol3)

        snap_vol_vmax = EMC_StorageVolume()
        snap_vol_vmax['name'] = self.data.test_snapshot_vmax['name']
        snap_vol_vmax['CreationClassName'] = 'Symm_StorageVolume'
        snap_vol_vmax['ElementName'] = self.data.test_snapshot_vmax['name']
        snap_vol_vmax['DeviceID'] = self.data.test_snapshot_vmax['id']
        snap_vol_vmax['SystemName'] = self.data.storage_system_vmax
        # Added vol to vol.path
        snap_vol_vmax['SystemCreationClassName'] = 'Symm_StorageSystem'
        snap_vol_vmax.path = snap_vol_vmax
        snap_vol_vmax.path.classname = snap_vol_vmax['CreationClassName']

        name3 = {}
        name3['classname'] = 'Clar_StorageVolume'
        keys3 = {}
        keys3['CreationClassName'] = 'Clar_StorageVolume'
        keys3['SystemName'] = self.data.storage_system
        keys3['DeviceID'] = snap_vol_vmax['DeviceID']
        keys3['SystemCreationClassName'] = 'Clar_StorageSystem'
        name3['keybindings'] = keys3
        snap_vol_vmax['provider_location'] = str(name3)

        vols.append(snap_vol_vmax)

        failed_snap_replica = EMC_StorageVolume()
        failed_snap_replica['name'] = self.data.failed_snapshot_replica['name']
        failed_snap_replica['CreationClassName'] = 'Clar_StorageVolume'
        failed_snap_replica['ElementName'] =\
            self.data.failed_snapshot_replica['name']
        failed_snap_replica['DeviceID'] =\
            self.data.failed_snapshot_replica['id']
        failed_snap_replica['SystemName'] = self.data.storage_system
        # Added vol to vol.path
        failed_snap_replica['SystemCreationClassName'] = 'Clar_StorageSystem'
        failed_snap_replica.path = failed_snap_replica
        failed_snap_replica.path.classname =\
            failed_snap_replica['CreationClassName']

        name4 = {}
        name4['classname'] = 'Clar_StorageVolume'
        keys4 = {}
        keys4['CreationClassName'] = 'Clar_StorageVolume'
        keys4['SystemName'] = self.data.storage_system
        keys4['DeviceID'] = failed_snap_replica['DeviceID']
        keys4['SystemCreationClassName'] = 'Clar_StorageSystem'
        name4['keybindings'] = keys4
        failed_snap_replica['provider_location'] = str(name4)

        vols.append(failed_snap_replica)

        failed_snap_sync = EMC_StorageVolume()
        failed_snap_sync['name'] = self.data.failed_snapshot_sync['name']
        failed_snap_sync['CreationClassName'] = 'Clar_StorageVolume'
        failed_snap_sync['ElementName'] =\
            self.data.failed_snapshot_sync['name']
        failed_snap_sync['DeviceID'] = self.data.failed_snapshot_sync['id']
        failed_snap_sync['SystemName'] = self.data.storage_system
        # Added vol to vol.path
        failed_snap_sync['SystemCreationClassName'] = 'Clar_StorageSystem'
        failed_snap_sync.path = failed_snap_sync
        failed_snap_sync.path.classname =\
            failed_snap_sync['CreationClassName']

        name5 = {}
        name5['classname'] = 'Clar_StorageVolume'
        keys5 = {}
        keys5['CreationClassName'] = 'Clar_StorageVolume'
        keys5['SystemName'] = self.data.storage_system
        keys5['DeviceID'] = failed_snap_sync['DeviceID']
        keys5['SystemCreationClassName'] = 'Clar_StorageSystem'
        name5['keybindings'] = keys5
        failed_snap_sync['provider_location'] = str(name5)

        vols.append(failed_snap_sync)

        failed_clone_rep = EMC_StorageVolume()
        failed_clone_rep['name'] = self.data.failed_clone_replica['name']
        failed_clone_rep['CreationClassName'] = 'Clar_StorageVolume'
        failed_clone_rep['ElementName'] =\
            self.data.failed_clone_replica['name']
        failed_clone_rep['DeviceID'] = self.data.failed_clone_replica['id']
        failed_clone_rep['SystemName'] = self.data.storage_system
        # Added vol to vol.path
        failed_clone_rep['SystemCreationClassName'] = 'Clar_StorageSystem'
        failed_clone_rep.path = failed_clone_rep
        failed_clone_rep.path.classname =\
            failed_clone_rep['CreationClassName']
        vols.append(failed_clone_rep)

        failed_clone_s = EMC_StorageVolume()
        failed_clone_s['name'] = self.data.failed_clone_sync['name']
        failed_clone_s['CreationClassName'] = 'Clar_StorageVolume'
        failed_clone_s['ElementName'] = self.data.failed_clone_sync['name']
        failed_clone_s['DeviceID'] = self.data.failed_clone_sync['id']
        failed_clone_s['SystemName'] = self.data.storage_system
        # Added vol to vol.path
        failed_clone_s['SystemCreationClassName'] = 'Clar_StorageSystem'
        failed_clone_s.path = failed_clone_s
        failed_clone_s.path.classname =\
            failed_clone_s['CreationClassName']
        vols.append(failed_clone_s)

        failed_delete_vol = EMC_StorageVolume()
        failed_delete_vol['name'] = 'failed_delete_vol'
        failed_delete_vol['CreationClassName'] = 'Clar_StorageVolume'
        failed_delete_vol['ElementName'] = 'failed_delete_vol'
        failed_delete_vol['DeviceID'] = '99999'
        failed_delete_vol['SystemName'] = self.data.storage_system
        # Added vol to vol.path
        failed_delete_vol['SystemCreationClassName'] = 'Clar_StorageSystem'
        failed_delete_vol.path = failed_delete_vol
        failed_delete_vol.path.classname =\
            failed_delete_vol['CreationClassName']
        vols.append(failed_delete_vol)

        failed_vol = EMC_StorageVolume()
        failed_vol['name'] = 'failed__vol'
        failed_vol['CreationClassName'] = 'Clar_StorageVolume'
        failed_vol['ElementName'] = 'failed_vol'
        failed_vol['DeviceID'] = '4'
        failed_vol['SystemName'] = self.data.storage_system
        # Added vol to vol.path
        failed_vol['SystemCreationClassName'] = 'Clar_StorageSystem'
        failed_vol.path = failed_vol
        failed_vol.path.classname =\
            failed_vol['CreationClassName']

        name_failed = {}
        name_failed['classname'] = 'Clar_StorageVolume'
        keys_failed = {}
        keys_failed['CreationClassName'] = 'Clar_StorageVolume'
        keys_failed['SystemName'] = self.data.storage_system
        keys_failed['DeviceID'] = failed_vol['DeviceID']
        keys_failed['SystemCreationClassName'] = 'Clar_StorageSystem'
        name_failed['keybindings'] = keys_failed
        failed_vol['provider_location'] = str(name_failed)

        vols.append(failed_vol)

        failed_extend_vol = EMC_StorageVolume()
        failed_extend_vol['name'] = 'failed_extend_vol'
        failed_extend_vol['CreationClassName'] = 'Clar_StorageVolume'
        failed_extend_vol['ElementName'] = 'failed_extend_vol'
        failed_extend_vol['DeviceID'] = '9'
        failed_extend_vol['SystemName'] = self.data.storage_system
        # Added vol to vol.path
        failed_extend_vol['SystemCreationClassName'] = 'Clar_StorageSystem'
        failed_extend_vol.path = failed_extend_vol
        failed_extend_vol.path.classname =\
            failed_extend_vol['CreationClassName']

        name_extend_failed = {}
        name_extend_failed['classname'] = 'Clar_StorageVolume'
        keys_extend_failed = {}
        keys_extend_failed['CreationClassName'] = 'Clar_StorageVolume'
        keys_extend_failed['SystemName'] = self.data.storage_system
        keys_extend_failed['DeviceID'] = failed_extend_vol['DeviceID']
        keys_extend_failed['SystemCreationClassName'] = 'Clar_StorageSystem'
        name_extend_failed['keybindings'] = keys_extend_failed
        failed_extend_vol['provider_location'] = str(name_extend_failed)

        vols.append(failed_extend_vol)

        return vols

    def _enum_syncsvsvs(self):
        syncs = []

        vols = self._enum_storagevolumes()

        sync = self._create_sync(vols[0], vols[1], 100)
        syncs.append(sync)

        sync2 = self._create_sync(vols[1], vols[2], 100)
        syncs.append(sync2)

        sync3 = self._create_sync(vols[0], vols[3], 100)
        syncs.append(sync3)

        objpath1 = vols[1]
        for vol in vols:
            if vol['ElementName'] == 'failed_snapshot_sync':
                objpath2 = vol
                break
        sync4 = self._create_sync(objpath1, objpath2, 100)
        syncs.append(sync4)

        objpath1 = vols[0]
        for vol in vols:
            if vol['ElementName'] == 'failed_clone_sync':
                objpath2 = vol
                break
        sync5 = self._create_sync(objpath1, objpath2, 100)
        syncs.append(sync5)

        return syncs

    def _create_sync(self, objpath1, objpath2, percentsynced):
        sync = {}
        sync['SyncedElement'] = objpath2
        sync['SystemElement'] = objpath1
        sync['CreationClassName'] = 'SE_StorageSynchronized_SV_SV'
        sync['PercentSynced'] = percentsynced
        return sync

    def _enum_unitnames(self):
        return self._ref_unitnames()

    def _enum_lunmaskctrls(self):
        ctrls = []
        ctrl = {}
        ctrl['CreationClassName'] = self.data.lunmask_creationclass
        ctrl['DeviceID'] = self.data.lunmaskctrl_id
        ctrl['SystemName'] = self.data.storage_system
        ctrls.append(ctrl)
        return ctrls

    def _enum_processors(self):
        ctrls = []
        ctrl = {}
        ctrl['CreationClassName'] = 'Clar_StorageProcessorSystem'
        ctrl['Name'] = self.data.storage_system + '+SP_A'
        ctrls.append(ctrl)
        return ctrls

    def _enum_hdwidmgmts(self):
        services = []
        srv = {}
        srv['SystemName'] = self.data.storage_system
        services.append(srv)
        return services

    def _enum_storhdwids(self):
        storhdwids = []
        hdwid = SE_StorageHardwareID()
        hdwid['StorageID'] = self.data.connector['wwpns'][0]

        hdwid.path = hdwid
        storhdwids.append(hdwid)
        return storhdwids

    def _default_enum(self):
        names = []
        name = {}
        name['Name'] = 'default'
        names.append(name)
        return names


class EMCSMISISCSIDriverTestCase(test.TestCase):

    def setUp(self):

        self.data = EMCSMISCommonData()

        self.tempdir = tempfile.mkdtemp()
        super(EMCSMISISCSIDriverTestCase, self).setUp()
        self.config_file_path = None
        self.create_fake_config_file()

        configuration = mock.Mock()
        configuration.cinder_emc_config_file = self.config_file_path

        self.stubs.Set(EMCSMISISCSIDriver, '_do_iscsi_discovery',
                       self.fake_do_iscsi_discovery)
        self.stubs.Set(EMCSMISCommon, '_get_ecom_connection',
                       self.fake_ecom_connection)
        instancename = FakeCIMInstanceName()
        self.stubs.Set(EMCSMISCommon, '_getinstancename',
                       instancename.fake_getinstancename)
        self.stubs.Set(time, 'sleep',
                       self.fake_sleep)
        driver = EMCSMISISCSIDriver(configuration=configuration)
        driver.db = FakeDB()
        self.driver = driver

    def create_fake_config_file(self):

        doc = Document()
        emc = doc.createElement("EMC")
        doc.appendChild(emc)

        storagetype = doc.createElement("StorageType")
        storagetypetext = doc.createTextNode("gold")
        emc.appendChild(storagetype)
        storagetype.appendChild(storagetypetext)

        ecomserverip = doc.createElement("EcomServerIp")
        ecomserveriptext = doc.createTextNode("1.1.1.1")
        emc.appendChild(ecomserverip)
        ecomserverip.appendChild(ecomserveriptext)

        ecomserverport = doc.createElement("EcomServerPort")
        ecomserverporttext = doc.createTextNode("10")
        emc.appendChild(ecomserverport)
        ecomserverport.appendChild(ecomserverporttext)

        ecomusername = doc.createElement("EcomUserName")
        ecomusernametext = doc.createTextNode("user")
        emc.appendChild(ecomusername)
        ecomusername.appendChild(ecomusernametext)

        ecompassword = doc.createElement("EcomPassword")
        ecompasswordtext = doc.createTextNode("pass")
        emc.appendChild(ecompassword)
        ecompassword.appendChild(ecompasswordtext)

        timeout = doc.createElement("Timeout")
        timeouttext = doc.createTextNode("0")
        emc.appendChild(timeout)
        timeout.appendChild(timeouttext)

        self.config_file_path = self.tempdir + '/' + self.data.config_file_name
        f = open(self.config_file_path, 'w')
        doc.writexml(f)
        f.close()

    def fake_ecom_connection(self):
        conn = FakeEcomConnection()
        return conn

    def fake_do_iscsi_discovery(self, volume):
        output = []
        item = '10.0.0.3:3260,1 iqn.1992-04.com.emc:cx.apm00123907237.a8'
        item2 = '10.0.0.4:3260,2 iqn.1992-04.com.emc:cx.apm00123907237.b8'
        output.append(item)
        output.append(item2)
        return output

    def fake_sleep(self, seconds):
        return

    def test_get_volume_stats(self):
        self.driver.get_volume_stats(True)

    def test_create_destroy(self):
        self.driver.create_volume(self.data.test_volume)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_snapshot_destroy(self):
        self.driver.create_volume(self.data.test_volume)
        self.driver.create_snapshot(self.data.test_snapshot)
        self.driver.create_volume_from_snapshot(
            self.data.test_clone, self.data.test_snapshot)
        self.driver.create_cloned_volume(
            self.data.test_clone3, self.data.test_volume)
        self.driver.delete_volume(self.data.test_clone)
        self.driver.delete_volume(self.data.test_clone3)
        self.driver.delete_snapshot(self.data.test_snapshot)
        self.driver.delete_volume(self.data.test_volume)

    def test_map_unmap(self):
        self.driver.create_volume(self.data.test_volume)
        self.data.test_volume['EMCCurrentOwningStorageProcessor'] = 'SP_A'
        connection_info = self.driver.initialize_connection(
            self.data.test_volume,
            self.data.connector)
        self.driver.terminate_connection(self.data.test_volume,
                                         self.data.connector)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_failed(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume,
                          self.data.test_failed_volume)

    def test_create_volume_snapshot_unsupported(self):
        self.driver.create_volume(self.data.test_volume)
        self.driver.create_snapshot(self.data.test_snapshot_vmax)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          self.data.test_clone,
                          self.data.test_snapshot_vmax)
        self.driver.delete_snapshot(self.data.test_snapshot_vmax)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_snapshot_replica_failed(self):
        self.driver.create_volume(self.data.test_volume)
        self.driver.create_snapshot(self.data.test_snapshot)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          self.data.failed_snapshot_replica,
                          self.data.test_snapshot)
        self.driver.delete_snapshot(self.data.test_snapshot)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_snapshot_sync_failed(self):
        self.driver.create_volume(self.data.test_volume)
        self.driver.create_snapshot(self.data.test_snapshot)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          self.data.failed_snapshot_sync,
                          self.data.test_snapshot)
        self.driver.delete_snapshot(self.data.test_snapshot)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_clone_replica_failed(self):
        self.driver.create_volume(self.data.test_volume)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          self.data.failed_clone_replica,
                          self.data.test_volume)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_clone_sync_failed(self):
        self.driver.create_volume(self.data.test_volume)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          self.data.failed_clone_sync,
                          self.data.test_volume)
        self.driver.delete_volume(self.data.test_volume)

    def test_delete_volume_notfound(self):
        notfound_delete_vol = {}
        notfound_delete_vol['name'] = 'notfound_delete_vol'
        notfound_delete_vol['id'] = '10'
        notfound_delete_vol['CreationClassName'] = 'Clar_StorageVolume'
        notfound_delete_vol['SystemName'] = self.data.storage_system
        notfound_delete_vol['DeviceID'] = notfound_delete_vol['id']
        notfound_delete_vol['SystemCreationClassName'] = 'Clar_StorageSystem'
        name = {}
        name['classname'] = 'Clar_StorageVolume'
        keys = {}
        keys['CreationClassName'] = notfound_delete_vol['CreationClassName']
        keys['SystemName'] = notfound_delete_vol['SystemName']
        keys['DeviceID'] = notfound_delete_vol['DeviceID']
        keys['SystemCreationClassName'] =\
            notfound_delete_vol['SystemCreationClassName']
        name['keybindings'] = keys
        notfound_delete_vol['provider_location'] = str(name)
        self.driver.delete_volume(notfound_delete_vol)

    def test_delete_volume_failed(self):
        self.driver.create_volume(self.data.failed_delete_vol)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.delete_volume,
                          self.data.failed_delete_vol)

    def test_extend_volume(self):
        self.driver.create_volume(self.data.test_volume)
        self.driver.extend_volume(self.data.test_volume, '10')
        self.driver.create_volume(self.data.failed_extend_vol)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.extend_volume,
                          self.data.failed_extend_vol,
                          '10')

    def _cleanup(self):
        bExists = os.path.exists(self.config_file_path)
        if bExists:
            os.remove(self.config_file_path)
        shutil.rmtree(self.tempdir)

    def tearDown(self):
        self._cleanup()
        super(EMCSMISISCSIDriverTestCase, self).tearDown()


class EMCSMISFCDriverTestCase(test.TestCase):

    def setUp(self):

        self.data = EMCSMISCommonData()

        self.tempdir = tempfile.mkdtemp()
        super(EMCSMISFCDriverTestCase, self).setUp()
        self.config_file_path = None
        self.create_fake_config_file()

        configuration = mock.Mock()
        configuration.cinder_emc_config_file = self.config_file_path

        self.stubs.Set(EMCSMISCommon, '_get_ecom_connection',
                       self.fake_ecom_connection)
        instancename = FakeCIMInstanceName()
        self.stubs.Set(EMCSMISCommon, '_getinstancename',
                       instancename.fake_getinstancename)
        self.stubs.Set(time, 'sleep',
                       self.fake_sleep)
        driver = EMCSMISFCDriver(configuration=configuration)
        driver.db = FakeDB()
        self.driver = driver

    def create_fake_config_file(self):

        doc = Document()
        emc = doc.createElement("EMC")
        doc.appendChild(emc)

        storagetype = doc.createElement("StorageType")
        storagetypetext = doc.createTextNode("gold")
        emc.appendChild(storagetype)
        storagetype.appendChild(storagetypetext)

        ecomserverip = doc.createElement("EcomServerIp")
        ecomserveriptext = doc.createTextNode("1.1.1.1")
        emc.appendChild(ecomserverip)
        ecomserverip.appendChild(ecomserveriptext)

        ecomserverport = doc.createElement("EcomServerPort")
        ecomserverporttext = doc.createTextNode("10")
        emc.appendChild(ecomserverport)
        ecomserverport.appendChild(ecomserverporttext)

        ecomusername = doc.createElement("EcomUserName")
        ecomusernametext = doc.createTextNode("user")
        emc.appendChild(ecomusername)
        ecomusername.appendChild(ecomusernametext)

        ecompassword = doc.createElement("EcomPassword")
        ecompasswordtext = doc.createTextNode("pass")
        emc.appendChild(ecompassword)
        ecompassword.appendChild(ecompasswordtext)

        timeout = doc.createElement("Timeout")
        timeouttext = doc.createTextNode("0")
        emc.appendChild(timeout)
        timeout.appendChild(timeouttext)

        self.config_file_path = self.tempdir + '/' + self.data.config_file_name
        f = open(self.config_file_path, 'w')
        doc.writexml(f)
        f.close()

    def fake_ecom_connection(self):
        conn = FakeEcomConnection()
        return conn

    def fake_sleep(self, seconds):
        return

    def test_get_volume_stats(self):
        self.driver.get_volume_stats(True)

    def test_create_destroy(self):
        self.data.test_volume['volume_type_id'] = None
        self.driver.create_volume(self.data.test_volume)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_snapshot_destroy(self):
        self.data.test_volume['volume_type_id'] = None
        self.driver.create_volume(self.data.test_volume)
        self.driver.create_snapshot(self.data.test_snapshot)
        self.driver.create_volume_from_snapshot(
            self.data.test_clone, self.data.test_snapshot)
        self.driver.create_cloned_volume(
            self.data.test_clone3, self.data.test_volume)
        self.driver.delete_volume(self.data.test_clone)
        self.driver.delete_volume(self.data.test_clone3)
        self.driver.delete_snapshot(self.data.test_snapshot)
        self.driver.delete_volume(self.data.test_volume)

    def test_map_unmap(self):
        self.data.test_volume['volume_type_id'] = None
        self.driver.create_volume(self.data.test_volume)

        output = {
            'driver_volume_type': 'fibre_channel',
            'data': {
                'target_lun': 0,
                'target_wwn': ['1234567890123', '0987654321321'],
                'target_discovered': True,
                'initiator_target_map': {'123456789012345':
                                         ['1234567890123', '0987654321321'],
                                         '123456789054321':
                                         ['1234567890123', '0987654321321'],
                                         }}}

        connection_info = self.driver.initialize_connection(
            self.data.test_volume,
            self.data.connector)
        self.assertEqual(connection_info, output)

        connection_info = self.driver.terminate_connection(
            self.data.test_volume,
            self.data.connector)

        # Verify calls in terminate_connection are executed
        conf_service = {}
        conf_service['SystemName'] = self.data.storage_system
        conf_service['CreationClassName'] =\
            self.data.ctrlconf_service_creationclass

        vol_instance = self.driver.common._find_lun(self.data.test_volume)

        expected = [
            mock.call._get_ecom_connection(),
            mock.call.find_device_number(self.data.test_volume),
            mock.call._find_lun(self.data.test_volume),
            mock.call.self._find_controller_configuration_service(
                self.data.storage_system),
            mock.call._remove_members(conf_service, vol_instance),
            mock.call.get_target_wwns(
                self.data.storage_system,
                self.data.connector)]

        output = {
            'driver_volume_type': 'fibre_channel',
            'data': {
                'target_wwn': ['1234567890123', '0987654321321'],
                'initiator_target_map': {'123456789012345':
                                         ['1234567890123', '0987654321321'],
                                         '123456789054321':
                                         ['1234567890123', '0987654321321'],
                                         }}}

        self.assertEqual(connection_info, output)

        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_failed(self):
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume,
                          self.data.test_failed_volume)

    def test_create_volume_snapshot_unsupported(self):
        self.data.test_volume['volume_type_id'] = None
        self.driver.create_volume(self.data.test_volume)
        self.driver.create_snapshot(self.data.test_snapshot_vmax)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          self.data.test_clone,
                          self.data.test_snapshot_vmax)
        self.driver.delete_snapshot(self.data.test_snapshot_vmax)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_snapshot_replica_failed(self):
        self.data.test_volume['volume_type_id'] = None
        self.driver.create_volume(self.data.test_volume)
        self.driver.create_snapshot(self.data.test_snapshot)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          self.data.failed_snapshot_replica,
                          self.data.test_snapshot)
        self.driver.delete_snapshot(self.data.test_snapshot)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_snapshot_sync_failed(self):
        self.data.test_volume['volume_type_id'] = None
        self.driver.create_volume(self.data.test_volume)
        self.driver.create_snapshot(self.data.test_snapshot)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_volume_from_snapshot,
                          self.data.failed_snapshot_sync,
                          self.data.test_snapshot)
        self.driver.delete_snapshot(self.data.test_snapshot)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_clone_replica_failed(self):
        self.data.test_volume['volume_type_id'] = None
        self.driver.create_volume(self.data.test_volume)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          self.data.failed_clone_replica,
                          self.data.test_volume)
        self.driver.delete_volume(self.data.test_volume)

    def test_create_volume_clone_sync_failed(self):
        self.data.test_volume['volume_type_id'] = None
        self.driver.create_volume(self.data.test_volume)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.create_cloned_volume,
                          self.data.failed_clone_sync,
                          self.data.test_volume)
        self.driver.delete_volume(self.data.test_volume)

    def test_delete_volume_notfound(self):
        notfound_delete_vol = {}
        notfound_delete_vol['name'] = 'notfound_delete_vol'
        notfound_delete_vol['id'] = '10'
        notfound_delete_vol['CreationClassName'] = 'Clar_StorageVolume'
        notfound_delete_vol['SystemName'] = self.data.storage_system
        notfound_delete_vol['DeviceID'] = notfound_delete_vol['id']
        notfound_delete_vol['SystemCreationClassName'] = 'Clar_StorageSystem'
        name = {}
        name['classname'] = 'Clar_StorageVolume'
        keys = {}
        keys['CreationClassName'] = notfound_delete_vol['CreationClassName']
        keys['SystemName'] = notfound_delete_vol['SystemName']
        keys['DeviceID'] = notfound_delete_vol['DeviceID']
        keys['SystemCreationClassName'] =\
            notfound_delete_vol['SystemCreationClassName']
        name['keybindings'] = keys
        notfound_delete_vol['provider_location'] = str(name)
        self.driver.delete_volume(notfound_delete_vol)

    def test_delete_volume_failed(self):
        self.driver.create_volume(self.data.failed_delete_vol)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.delete_volume,
                          self.data.failed_delete_vol)

    def test_extend_volume(self):
        self.data.test_volume['volume_type_id'] = None
        self.driver.create_volume(self.data.test_volume)
        self.driver.extend_volume(self.data.test_volume, '10')
        self.driver.create_volume(self.data.failed_extend_vol)
        self.assertRaises(exception.VolumeBackendAPIException,
                          self.driver.extend_volume,
                          self.data.failed_extend_vol,
                          '10')

    @mock.patch.object(
        volume_types,
        'get_volume_type_extra_specs',
        return_value={'storagetype:pool': 'gold',
                      'storagetype:provisioning': 'thick'})
    def test_create_volume_with_volume_type(self, _mock_volume_type):
        volume_with_vt = self.data.test_volume
        volume_with_vt['volume_type_id'] = 1
        self.driver.create_volume(volume_with_vt)

        configservice = {'CreationClassName':
                         'Clar_StorageConfigurationService',
                         'SystemName': 'CLARiiON+APM00123456789'}

        pool = {'InstanceID': 'CLARiiON+APM00123456789+U+gold',
                'CreationClassName': 'Clar_UnifiedStoragePool'}

        volumesize = int(volume_with_vt['size']) * units.GiB

        storage_type = {'storagetype:provisioning': 'thick',
                        'storagetype:pool': 'gold'}

        expected = [
            mock.call._get_storage_type(volume_with_vt),
            mock.call._find_pool('gold'),
            mock.call.get_provisioning(storage_type),
            mock.call.InvokeMethod('CreateOrModifyElementFromStoragePool',
                                   configservice, volume_with_vt['name'],
                                   pool,
                                   self.driver.common._getnum(2, '16'),
                                   self.driver.common._getnum(volumesize,
                                                              '64'))]

    def _cleanup(self):
        bExists = os.path.exists(self.config_file_path)
        if bExists:
            os.remove(self.config_file_path)
        shutil.rmtree(self.tempdir)

    def tearDown(self):
        self._cleanup()
        super(EMCSMISFCDriverTestCase, self).tearDown()



# Copyright (c) 2013 VMware, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Classes to handle image files.
Collection of classes to handle image upload/download to/from Image service
(like Glance image storage and retrieval service) from/to VMware server.
"""

import httplib
import netaddr
import urllib
import urllib2

import six.moves.urllib.parse as urlparse

from cinder.openstack.common import log as logging
from cinder.volume.drivers.vmware import error_util
from cinder.volume.drivers.vmware import vim_util

LOG = logging.getLogger(__name__)
USER_AGENT = 'OpenStack-ESX-Adapter'
READ_CHUNKSIZE = 65536


class GlanceFileRead(object):
    """Glance file read handler class."""

    def __init__(self, glance_read_iter):
        self.glance_read_iter = glance_read_iter
        self.iter = self.get_next()

    def read(self, chunk_size):
        """Read an item from the queue.

        The chunk size is ignored for the Client ImageBodyIterator
        uses its own CHUNKSIZE.
        """
        try:
            return self.iter.next()
        except StopIteration:
            return ""

    def get_next(self):
        """Get the next item from the image iterator."""
        for data in self.glance_read_iter:
            yield data

    def close(self):
        """A dummy close just to maintain consistency."""
        pass


class VMwareHTTPFile(object):
    """Base class for VMDK file access over HTTP."""

    def __init__(self, file_handle):
        self.eof = False
        self.file_handle = file_handle

    def close(self):
        """Close the file handle."""
        try:
            self.file_handle.close()
        except Exception as exc:
            LOG.exception(exc)

    def __del__(self):
        """Close the file handle on garbage collection."""
        self.close()

    def _build_vim_cookie_headers(self, vim_cookies):
        """Build ESX host session cookie headers."""
        cookie_header = ""
        for vim_cookie in vim_cookies:
            cookie_header = vim_cookie.name + '=' + vim_cookie.value
            break
        return cookie_header

    def write(self, data):
        """Write data to the file."""
        raise NotImplementedError()

    def read(self, chunk_size):
        """Read a chunk of data."""
        raise NotImplementedError()

    def get_size(self):
        """Get size of the file to be read."""
        raise NotImplementedError()

    def _is_valid_ipv6(self, address):
        """Whether given host address is a valid IPv6 address."""
        try:
            return netaddr.valid_ipv6(address)
        except Exception:
            return False

    def get_soap_url(self, scheme, host):
        """return IPv4/v6 compatible url constructed for host."""
        if self._is_valid_ipv6(host):
            return '%s://[%s]' % (scheme, host)
        return '%s://%s' % (scheme, host)

    def _fix_esx_url(self, url, host):
        """Fix netloc if it is a ESX host.

        For a ESX host the netloc is set to '*' in the url returned in
        HttpNfcLeaseInfo. The netloc is right IP when talking to a VC.
        """
        urlp = urlparse.urlparse(url)
        if urlp.netloc == '*':
            scheme, _, path, params, query, fragment = urlp
            url = urlparse.urlunparse((scheme, host, path, params,
                                       query, fragment))
        return url

    def find_vmdk_url(self, lease_info, host):
        """Find the URL corresponding to a vmdk disk in lease info."""
        url = None
        for deviceUrl in lease_info.deviceUrl:
            if deviceUrl.disk:
                url = self._fix_esx_url(deviceUrl.url, host)
                break
        return url


class VMwareHTTPWriteFile(VMwareHTTPFile):
    """VMware file write handler class."""

    def __init__(self, host, data_center_name, datastore_name, cookies,
                 file_path, file_size, scheme='https'):
        soap_url = self.get_soap_url(scheme, host)
        base_url = '%s/folder/%s' % (soap_url, file_path)
        param_list = {'dcPath': data_center_name, 'dsName': datastore_name}
        base_url = base_url + '?' + urllib.urlencode(param_list)
        _urlparse = urlparse.urlparse(base_url)
        scheme, netloc, path, params, query, fragment = _urlparse
        if scheme == 'http':
            conn = httplib.HTTPConnection(netloc)
        elif scheme == 'https':
            conn = httplib.HTTPSConnection(netloc)
        conn.putrequest('PUT', path + '?' + query)
        conn.putheader('User-Agent', USER_AGENT)
        conn.putheader('Content-Length', file_size)
        conn.putheader('Cookie', self._build_vim_cookie_headers(cookies))
        conn.endheaders()
        self.conn = conn
        VMwareHTTPFile.__init__(self, conn)

    def write(self, data):
        """Write to the file."""
        self.file_handle.send(data)

    def close(self):
        """Get the response and close the connection."""
        try:
            self.conn.getresponse()
        except Exception as excep:
            LOG.debug(_("Exception during HTTP connection close in "
                        "VMwareHTTPWrite. Exception is %s.") % excep)
        super(VMwareHTTPWriteFile, self).close()


class VMwareHTTPWriteVmdk(VMwareHTTPFile):
    """Write VMDK over HTTP using VMware HttpNfcLease."""

    def __init__(self, session, host, rp_ref, vm_folder_ref, vm_create_spec,
                 vmdk_size):
        """Initialize a writer for vmdk file.

        :param session: a valid api session to ESX/VC server
        :param host: the ESX or VC host IP
        :param rp_ref: resource pool into which backing VM is imported
        :param vm_folder_ref: VM folder in ESX/VC inventory to use as parent
               of backing VM
        :param vm_create_spec: backing VM created using this create spec
        :param vmdk_size: VMDK size to be imported into backing VM
        """
        self._session = session
        self._vmdk_size = vmdk_size
        self._progress = 0
        lease = session.invoke_api(session.vim, 'ImportVApp', rp_ref,
                                   spec=vm_create_spec, folder=vm_folder_ref)
        session.wait_for_lease_ready(lease)
        self._lease = lease
        lease_info = session.invoke_api(vim_util, 'get_object_property',
                                        session.vim, lease, 'info')
        # Find the url for vmdk device
        url = self.find_vmdk_url(lease_info, host)
        if not url:
            msg = _("Could not retrieve URL from lease.")
            LOG.exception(msg)
            raise error_util.VimException(msg)
        LOG.info(_("Opening vmdk url: %s for write.") % url)

        # Prepare the http connection to the vmdk url
        cookies = session.vim.client.options.transport.cookiejar
        _urlparse = urlparse.urlparse(url)
        scheme, netloc, path, params, query, fragment = _urlparse
        if scheme == 'http':
            conn = httplib.HTTPConnection(netloc)
        elif scheme == 'https':
            conn = httplib.HTTPSConnection(netloc)
        if query:
            path = path + '?' + query
        conn.putrequest('PUT', path)
        conn.putheader('User-Agent', USER_AGENT)
        conn.putheader('Content-Length', str(vmdk_size))
        conn.putheader('Overwrite', 't')
        conn.putheader('Cookie', self._build_vim_cookie_headers(cookies))
        conn.putheader('Content-Type', 'binary/octet-stream')
        conn.endheaders()
        self.conn = conn
        VMwareHTTPFile.__init__(self, conn)

    def write(self, data):
        """Write to the file."""
        self._progress += len(data)
        LOG.debug(_("Written %s bytes to vmdk.") % self._progress)
        self.file_handle.send(data)

    def update_progress(self):
        """Updates progress to lease.

        This call back to the lease is essential to keep the lease alive
        across long running write operations.
        """
        percent = int(float(self._progress) / self._vmdk_size * 100)
        try:
            LOG.debug(_("Updating progress to %s percent.") % percent)
            self._session.invoke_api(self._session.vim,
                                     'HttpNfcLeaseProgress',
                                     self._lease, percent=percent)
        except error_util.VimException as ex:
            LOG.exception(ex)
            raise ex

    def close(self):
        """End the lease and close the connection."""
        state = self._session.invoke_api(vim_util, 'get_object_property',
                                         self._session.vim,
                                         self._lease, 'state')
        if state == 'ready':
            self._session.invoke_api(self._session.vim, 'HttpNfcLeaseComplete',
                                     self._lease)
            LOG.debug(_("Lease released."))
        else:
            LOG.debug(_("Lease is already in state: %s.") % state)
        super(VMwareHTTPWriteVmdk, self).close()


class VMwareHTTPReadVmdk(VMwareHTTPFile):
    """read VMDK over HTTP using VMware HttpNfcLease."""

    def __init__(self, session, host, vm_ref, vmdk_path, vmdk_size):
        """Initialize a writer for vmdk file.

        During an export operation the vmdk disk is converted to a
        stream-optimized sparse disk format. So the size of the VMDK
        after export may be smaller than the current vmdk disk size.

        :param session: a valid api session to ESX/VC server
        :param host: the ESX or VC host IP
        :param vm_ref: backing VM whose vmdk is to be exported
        :param vmdk_path: datastore relative path to vmdk file to be exported
        :param vmdk_size: current disk size of vmdk file to be exported
        """
        self._session = session
        self._vmdk_size = vmdk_size
        self._progress = 0
        lease = session.invoke_api(session.vim, 'ExportVm', vm_ref)
        session.wait_for_lease_ready(lease)
        self._lease = lease
        lease_info = session.invoke_api(vim_util, 'get_object_property',
                                        session.vim, lease, 'info')

        # find the right disk url corresponding to given vmdk_path
        url = self.find_vmdk_url(lease_info, host)
        if not url:
            msg = _("Could not retrieve URL from lease.")
            LOG.exception(msg)
            raise error_util.VimException(msg)
        LOG.info(_("Opening vmdk url: %s for read.") % url)

        cookies = session.vim.client.options.transport.cookiejar
        headers = {'User-Agent': USER_AGENT,
                   'Cookie': self._build_vim_cookie_headers(cookies)}
        request = urllib2.Request(url, None, headers)
        conn = urllib2.urlopen(request)
        VMwareHTTPFile.__init__(self, conn)

    def read(self, chunk_size):
        """Read a chunk from file."""
        data = self.file_handle.read(READ_CHUNKSIZE)
        self._progress += len(data)
        LOG.debug("Read %s bytes from vmdk." % self._progress)
        return data

    def update_progress(self):
        """Updates progress to lease.

        This call back to the lease is essential to keep the lease alive
        across long running read operations.
        """
        percent = int(float(self._progress) / self._vmdk_size * 100)
        try:
            LOG.debug(_("Updating progress to %s percent.") % percent)
            self._session.invoke_api(self._session.vim,
                                     'HttpNfcLeaseProgress',
                                     self._lease, percent=percent)
        except error_util.VimException as ex:
            LOG.exception(ex)
            raise ex

    def close(self):
        """End the lease and close the connection."""
        state = self._session.invoke_api(vim_util, 'get_object_property',
                                         self._session.vim,
                                         self._lease, 'state')
        if state == 'ready':
            self._session.invoke_api(self._session.vim, 'HttpNfcLeaseComplete',
                                     self._lease)
            LOG.debug(_("Lease released."))
        else:
            LOG.debug(_("Lease is already in state: %s.") % state)
        super(VMwareHTTPReadVmdk, self).close()
